package com.xjrsoft.common.runner;

import com.xjrsoft.common.constant.GlobalConstant;
import com.xjrsoft.common.utils.RedisUtil;
import com.xjrsoft.module.system.entity.DictionaryDetail;
import com.xjrsoft.module.system.entity.DictionaryItem;
import com.xjrsoft.module.system.service.IDictionarydetailService;
import com.xjrsoft.module.system.service.IDictionaryitemService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.CommandLineRunner;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * 默认缓存数据字典
 *
 * @Author: tzx
 * @Date: 2023/1/11 11:14
 */
@Component
@AllArgsConstructor
@Slf4j
public class DicCacheRunner implements CommandLineRunner {

    private RedisUtil redisUtil;

    private IDictionaryitemService dictionaryitemService;

    private IDictionarydetailService dictionarydetailService;


    @Override
    public void run(String... args) {
        loadDicItemCache();
        loadDicDetailCache();
    }

    @Async
    void loadDicItemCache() {
        log.info("XJRSOFT: 加载所有数据字典项目缓存开始");
        List<DictionaryItem> list = dictionaryitemService.list();
        redisUtil.set(GlobalConstant.DIC_ITEM_CACHE_KEY, list);
        log.info("XJRSOFT: 加载所有数据字典项目缓存结束");
    }

    @Async
    void loadDicDetailCache() {
        log.info("XJRSOFT: 加载所有数据字典详情缓存开始");
        List<DictionaryDetail> list = dictionarydetailService.list();
        redisUtil.set(GlobalConstant.DIC_DETAIL_CACHE_KEY, list);
        log.info("XJRSOFT: 加载所有数据字典详情缓存结束");
    }

}
