package com.xjrsoft.common.utils;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import org.apache.commons.lang3.StringUtils;

import java.time.*;
import java.time.format.DateTimeFormatter;

public final class LocalDateTimeUtil {
    private LocalDateTimeUtil(){}

    public final static String LOCAL_TIME_FORMAT = "HH:mm:ss";

    public final static String LOCAL_DATE_TIME_FORMAT = "yyyy-MM-dd HH:mm:ss";

    public final static String LOCAL_DATE_FORMAT = "yyyy-MM-dd";

    public final static String YEAR_MONTH_FORMAT = "yyyy-MM";

    public final static String YEAR_FORMAT = "yyyy";

    public static LocalTime parseTime(String value) {
        return parseTime(value, LOCAL_TIME_FORMAT);
    }

    public static LocalTime parseTime(String value, String format) {
        if (StrUtil.isBlank(value)) {
            return null;
        }
        return LocalTime.parse(value, DateTimeFormatter.ofPattern(format));
    }

    public static LocalDateTime parseDate(String value) {
        return parseDate(value, LOCAL_DATE_TIME_FORMAT);
    }

    public static LocalDateTime parseDate(String value, String format) {
        if (StrUtil.isBlank(value)) {
            return null;
        }
        // 根据format切掉多余的时间
        value = StringUtils.substring(value, 0, format.length());
        LocalDateTime localDateTime = null;
        format = convertFontFormat(format);
        DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern(format);
        switch (format) {
            case YEAR_FORMAT:
                Year year = Year.parse(value, dateTimeFormatter);
                localDateTime =  LocalDateTime.from(year.atDay(1).atStartOfDay());
                break;
            case YEAR_MONTH_FORMAT:
                YearMonth yearMonth = YearMonth.parse(value, dateTimeFormatter);
                localDateTime = LocalDateTime.from(yearMonth.atDay(1).atStartOfDay());
                break;
            case LOCAL_DATE_FORMAT:
                LocalDate localDate = LocalDate.parse(value, dateTimeFormatter);
                localDateTime = LocalDateTime.from(localDate.atStartOfDay());
                break;
            default:
                localDateTime = LocalDateTime.parse(value, dateTimeFormatter);
                break;
        }
        return localDateTime;
    }

    public static String convertFontFormat(String fontFormat) {
        if (StrUtil.isEmpty(fontFormat)) {
            return fontFormat;
        }
        return fontFormat.replace("YYYY", "yyyy").replace("DD", "dd");
    }

    public static String format(LocalDateTime dateTime, String format) {
        if (dateTime == null) {
            return StringPool.EMPTY;
        }
        return dateTime.format(DateTimeFormatter.ofPattern(format));
    }

    public static String format(LocalTime time) {
        if (time == null) {
            return StringPool.EMPTY;
        }
        return time.format(DateTimeFormatter.ofPattern(LOCAL_TIME_FORMAT));
    }
}
