package com.xjrsoft.module.courseTable.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.yulichang.wrapper.MPJLambdaWrapper;
import com.xjrsoft.common.enums.TimeNumberEnum;
import com.xjrsoft.common.enums.TimePeriodEnum;
import com.xjrsoft.common.enums.WeekEnum;
import com.xjrsoft.common.exception.MyException;
import com.xjrsoft.common.utils.VoToColumnUtil;
import com.xjrsoft.config.TimetableConfig;
import com.xjrsoft.module.app.entity.AppPageDesign;
import com.xjrsoft.module.app.vo.AppPageDesignVo;
import com.xjrsoft.module.base.entity.BaseClass;
import com.xjrsoft.module.base.entity.BaseSemester;
import com.xjrsoft.module.base.service.IBaseClassService;
import com.xjrsoft.module.base.service.IBaseSemesterService;
import com.xjrsoft.module.courseTable.dto.ClassListDto;
import com.xjrsoft.module.courseTable.dto.CourseTableParse;
import com.xjrsoft.module.courseTable.entity.ClassTime;
import com.xjrsoft.module.courseTable.entity.CourseTable;
import com.xjrsoft.module.courseTable.mapper.ClassTimeMapper;
import com.xjrsoft.module.courseTable.mapper.CourseTableMapper;
import com.xjrsoft.module.courseTable.service.ICourseTableService;
import com.xjrsoft.module.courseTable.vo.ClassListVo;
import com.xjrsoft.module.personnel.entity.CarManagement;
import com.xjrsoft.module.system.entity.DictionaryDetail;
import com.xjrsoft.module.teacher.entity.BaseTeacher;
import com.xjrsoft.module.teacher.entity.XjrUser;
import com.xjrsoft.module.teacher.mapper.BaseTeacherMapper;
import com.xjrsoft.module.teacher.mapper.XjrUserMapper;
import com.xjrsoft.module.teacher.service.ITeacherbaseManagerService;
import com.xjrsoft.module.teacher.vo.XjrUserPageVo;
import lombok.AllArgsConstructor;
import org.apache.poi.xwpf.usermodel.*;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.io.InputStream;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 课表 服务实现类
 * </p>
 *
 * @author baomidou
 * @since 2023-09-02 02:19:56
 */
@Service
@AllArgsConstructor
public class CourseTableServiceImpl extends ServiceImpl<CourseTableMapper, CourseTable> implements ICourseTableService {

    private final CourseTableMapper courseTableMapper;

    private final IBaseClassService baseClassService;

    private final ITeacherbaseManagerService teacherbaseManagerService;

    private final IBaseSemesterService baseSemesterService;

    private final TimetableConfig timetableConfig;

    private final ClassTimeMapper classTimeMapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean wordImport(InputStream inputStream) throws IOException {
        List<CourseTableParse> parses = courseTableWordParses(inputStream);


        String semester = null;
        if (parses.size() > 0) {
            semester = parses.get(0).getSemester();
        }

        BaseSemester baseSemester = baseSemesterService.getOne(Wrappers.<BaseSemester>query().lambda().eq(BaseSemester::getName, semester));
        if (baseSemester == null) {
            throw new MyException(String.format("学期【%s】不存在", semester));
        }

        // 获取班级信息
        List<BaseClass> baseClassList = baseClassService.list();
        Map<String, BaseClass> baseClassMap = baseClassList.stream().collect(Collectors.toMap(BaseClass::getName, a -> a, (k1, k2) -> k1));

        // 获取教师信息
        MPJLambdaWrapper<XjrUser> queryWrapper = new MPJLambdaWrapper<>();
        queryWrapper
                .disableSubLogicDel()
                .innerJoin(BaseTeacher.class, BaseTeacher::getUserId, XjrUser::getId)
                .selectAll(XjrUser.class);
        List<XjrUser> xjrUserList = teacherbaseManagerService.list(queryWrapper);
        Map<String, XjrUser> xjrUsersMap = xjrUserList.stream().collect(Collectors.toMap(XjrUser::getName, a -> a, (k1, k2) -> k1));

        // 清空所有数据
        courseTableMapper.delete(Wrappers.<CourseTable>query().lambda().eq(CourseTable::getBaseSemesterId, baseSemester.getId()));

        List<CourseTable> params = new ArrayList<>();
        for (CourseTableParse item : parses) {
            CourseTable courseTable = BeanUtil.toBean(item, CourseTable.class);
            courseTable.setBaseSemesterId(baseSemester.getId());
            BaseClass baseClass = baseClassMap.get(item.getClassName());
            if (baseClass != null) {
                courseTable.setClassId(baseClass.getId());
            }
            XjrUser xjrUser = xjrUsersMap.get(item.getTeacherName());
            if (xjrUser != null) {
                courseTable.setTeacherId(xjrUser.getId());
            }
            if (item.getWeeksCn() != null) {
                courseTable.setWeeks(WeekEnum.getCode(item.getWeeksCn()));
            }
            params.add(courseTable);
        }
        return this.saveBatch(params);
    }

    @Override
    public List<ClassListVo> classList(ClassListDto dto) {
        LambdaQueryWrapper<CourseTable> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(CourseTable::getClassId, dto.getClassId())
                .eq(ObjectUtil.isNotEmpty(dto.getWeeks()), CourseTable::getWeeks,dto.getWeeks());

        List<CourseTable> courseTables = courseTableMapper.selectList(queryWrapper);

        List<ClassListVo> classListVos = BeanUtil.copyToList(courseTables, ClassListVo.class);
        Boolean isCurrentSummer = isCurrentSummer();

        Map<String, ClassTime> classTimeMap = getClassTimeMap();

        // 获取节次时间
        for (int i = 0; i < classListVos.size(); i++) {
            ClassListVo c = classListVos.get(i);
            String key = String.format("%d_%d", c.getTimePeriod(), c.getTimeNumber());
            ClassTime ct = classTimeMap.get(key);
            if (ct != null) {
                if (isCurrentSummer) {
                    c.setStartTime(ct.getSummerStartTime());
                    c.setEndTime(ct.getSummerEndTime());
                } else {
                    c.setStartTime(ct.getWinterStartTime());
                    c.setEndTime(ct.getWinterEndTime());
                }
            }
        }
        return classListVos;
    }

    /**
     * 获取节次
     *
     * @return
     */
    private Map<String, ClassTime> getClassTimeMap() {
        List<ClassTime> classTimes = classTimeMapper.selectList(null);

        Map<String, ClassTime> result = new HashMap<>();

        for (ClassTime item : classTimes) {
            String key = String.format("%d_%d", item.getTimePeriod(), item.getNumber());
            result.put(key, item);
        }

        return result;
    }

    /**
     * 当前时间是否在夏季
     *
     * @return
     */
    private Boolean isCurrentSummer() {
        DateTime now = DateUtil.date();
        DateTime summerStart = DateUtil.parse(String.format("%d-%s", now.year(), timetableConfig.getSummerStart()));
        DateTime summerEnd = DateUtil.parse(String.format("%d-%s", now.year(), timetableConfig.getSummerEnd()));
        DateTime winterStart = DateUtil.parse(String.format("%d-%s", now.year(), timetableConfig.getWinterStart()));
        DateTime winterEnd = DateUtil.parse(String.format("%d-%s", now.year(), timetableConfig.getWinterEnd()));

        // 如果开始时间大于结束时间，开始时间增加1年
        if (DateUtil.compare(summerStart, summerEnd) > 0) {
            summerEnd = DateUtil.offset(summerEnd, DateField.DAY_OF_YEAR, 1);
        }
        if (DateUtil.compare(winterStart, winterEnd) > 0) {
            winterEnd = DateUtil.offset(winterEnd, DateField.DAY_OF_YEAR, 1);
        }

        if (DateUtil.compare(summerStart, now) > 0 && DateUtil.compare(summerEnd, now) < 0) {
            return true;
        }
        return false;
    }

    /**
     * 解析word课表
     *
     * @param inputStream
     * @return
     * @throws IOException
     */
    private List<CourseTableParse> courseTableWordParses(InputStream inputStream) throws IOException {
        XWPFDocument doc = new XWPFDocument(inputStream);

        List<XWPFParagraph> paras = doc.getParagraphs();
        String semester = null;
        List<String> cNames = new ArrayList<>();
        //获取标题
        for (int i = 0; i < paras.size(); i++) {
            String txt = paras.get(i).getText();
            if (i == 0) {
                semester = txt;
                continue;
            }
            txt = txt.replaceAll("总课程表", "").replace("\n", "").trim();
            if (!txt.equals("") && !txt.equals(semester)) {
                cNames.add(txt);
            }
        }
        List<CourseTableParse> result = new ArrayList<>();
        //获取文档中所有的表格
        List<XWPFTable> tables = doc.getTables();
        int tNum = 0;
        for (XWPFTable table : tables) {
            int rNum = 0;
            String timePeriod = null;
            List<String> weeks = new ArrayList<>();
            List<XWPFTableRow> rows = table.getRows();
            for (XWPFTableRow row : rows) {
                //获取行对应的单元格
                List<XWPFTableCell> cells = row.getTableCells();
                String timeNumber = null;
                for (int i = 0; i < cells.size(); i++) {
                    String cellText = cells.get(i).getText();
                    if (cellText.equals("") || rNum < 1) continue;

                    if (rNum == 1) {
                        weeks.add(cellText);
                        continue;
                    }
                    if (i == 0) {
                        timePeriod = cellText;
                        continue;
                    }
                    if (i == 1) {
                        timeNumber = cellText;
                        continue;
                    }
                    List<XWPFParagraph> cParagraph = cells.get(i).getParagraphs();
                    CourseTableParse item = new CourseTableParse();
                    item.setSemester(semester);
                    item.setTimePeriod(TimePeriodEnum.getCode(timePeriod));
                    item.setTimeNumber(TimeNumberEnum.getCode(timeNumber));
                    String week = weeks.get(Math.max(i - 2, 0));
                    item.setWeeksCn(week);
                    item.setClassName(cNames.get(tNum));
                    for (int j = 0; j < cParagraph.size(); j++) {
                        cellText = cParagraph.get(j).getText().trim();
                        switch (j) {
                            case 0:
                                item.setCourseName(cellText);
                                break;
                            case 1:
                                item.setTeacherName(cellText);
                                break;
                            case 2:
                                item.setSiteName(cellText);
                                break;
                        }
                    }
                    result.add(item);
                }
                rNum++;
            }
            tNum++;
        }
        inputStream.close();
        return result;
    }
}
