package com.xjrsoft.module.erpModel.caseErpPurchase.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.github.yulichang.base.MPJBaseServiceImpl;
import com.xjrsoft.module.erpModel.caseErpPurchase.entity.CaseErpApply;
import com.xjrsoft.module.erpModel.caseErpPurchase.entity.CaseErpApplyDetail;
import com.xjrsoft.module.erpModel.caseErpPurchase.mapper.CaseErpApplyDetailMapper;
import com.xjrsoft.module.erpModel.caseErpPurchase.mapper.CaseErpApplyMapper;
import com.xjrsoft.module.erpModel.caseErpPurchase.service.ICaseErpPurchaseApplyService;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
* @title: service
* @Author 管理员
* @Date: 2023-07-20
* @Version 1.0
*/
@Service
@AllArgsConstructor
public class CaseErpPurchaseApplyServiceImpl extends MPJBaseServiceImpl<CaseErpApplyMapper, CaseErpApply> implements ICaseErpPurchaseApplyService {
    private final CaseErpApplyMapper purchasingApplyCaseErpApplyMapper;

    private final CaseErpApplyDetailMapper purchasingApplyCaseErpApplyDetailMapper;


    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean add(CaseErpApply caseErpApply) {
        purchasingApplyCaseErpApplyMapper.insert(caseErpApply);
        for (CaseErpApplyDetail caseErpApplyDetail : caseErpApply.getCaseErpApplyDetailList()) {
            caseErpApplyDetail.setApplyId(caseErpApply.getId());
            purchasingApplyCaseErpApplyDetailMapper.insert(caseErpApplyDetail);
        }

        return true;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean update(CaseErpApply caseErpApply) {
        purchasingApplyCaseErpApplyMapper.updateById(caseErpApply);
        //********************************* CaseErpApplyDetail  增删改  开始 *******************************************/
        {
            // 查出所有子级的id
            List<CaseErpApplyDetail> caseErpApplyDetailList = purchasingApplyCaseErpApplyDetailMapper.selectList(Wrappers.lambdaQuery(CaseErpApplyDetail.class).eq(CaseErpApplyDetail::getApplyId, caseErpApply.getId()).select(CaseErpApplyDetail::getId));
            List<Long> caseErpApplyDetailIds = caseErpApplyDetailList.stream().map(CaseErpApplyDetail::getId).collect(Collectors.toList());
            //原有子表单 没有被删除的主键
            List<Long> caseErpApplyDetailOldIds = caseErpApply.getCaseErpApplyDetailList().stream().map(CaseErpApplyDetail::getId).filter(Objects::nonNull).collect(Collectors.toList());
            //找到需要删除的id
            List<Long> caseErpApplyDetailRemoveIds = caseErpApplyDetailIds.stream().filter(item -> !caseErpApplyDetailOldIds.contains(item)).collect(Collectors.toList());

            for (CaseErpApplyDetail caseErpApplyDetail : caseErpApply.getCaseErpApplyDetailList()) {
                //如果不等于空则修改
                if (caseErpApplyDetail.getId() != null) {
                    purchasingApplyCaseErpApplyDetailMapper.updateById(caseErpApplyDetail);
                }
                //如果等于空 则新增
                else {
                    //已经不存在的id 删除
                    caseErpApplyDetail.setApplyId(caseErpApply.getId());
                    purchasingApplyCaseErpApplyDetailMapper.insert(caseErpApplyDetail);
                }
            }
            //已经不存在的id 删除
            if(caseErpApplyDetailRemoveIds.size() > 0){
                purchasingApplyCaseErpApplyDetailMapper.deleteBatchIds(caseErpApplyDetailRemoveIds);
            }
        }
        //********************************* CaseErpApplyDetail  增删改  结束 *******************************************/

        return true;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean delete(List<Long> ids) {
        purchasingApplyCaseErpApplyMapper.deleteBatchIds(ids);
        purchasingApplyCaseErpApplyDetailMapper.delete(Wrappers.lambdaQuery(CaseErpApplyDetail.class).in(CaseErpApplyDetail::getApplyId, ids));

        return true;
    }
}
