package com.xjrsoft.module.erpModel.caseErpStore.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.lang.TypeReference;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.xjrsoft.common.annotation.XjrLog;
import com.xjrsoft.common.constant.GlobalConstant;
import com.xjrsoft.common.enums.YesOrNoEnum;
import com.xjrsoft.common.model.result.R;
import com.xjrsoft.common.page.ConventPage;
import com.xjrsoft.common.page.PageOutput;
import com.xjrsoft.common.utils.RedisUtil;
import com.xjrsoft.common.utils.VoToColumnUtil;
import com.xjrsoft.module.erpModel.caseErpMaterial.entity.CaseErpMaterial;
import com.xjrsoft.module.erpModel.caseErpMaterial.service.ICaseErpMaterialService;
import com.xjrsoft.module.erpModel.caseErpStore.dto.AddCaseErpStoreReceiptDto;
import com.xjrsoft.module.erpModel.caseErpStore.dto.CaseErpStoreReceiptPageDto;
import com.xjrsoft.module.erpModel.caseErpStore.dto.UpdateCaseErpStoreReceiptDto;
import com.xjrsoft.module.erpModel.caseErpStore.entity.CaseErpStoreReceipt;
import com.xjrsoft.module.erpModel.caseErpStore.entity.CaseErpStoreReceiptDetail;
import com.xjrsoft.module.erpModel.caseErpStore.service.ICaseErpStoreReceiptDetailService;
import com.xjrsoft.module.erpModel.caseErpStore.service.ICaseErpStoreReceiptService;
import com.xjrsoft.module.erpModel.caseErpStore.vo.CaseErpStoreReceiptInfoVo;
import com.xjrsoft.module.erpModel.caseErpStore.vo.CaseErpStoreReceiptPageVo;
import com.xjrsoft.module.organization.entity.User;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 出入库单据信息表【case_erp_store_receipt】 前端控制器
 * </p>
 *
 * @author hnyyzy
 * @since 2023-07-20
 */
@RestController
@RequestMapping(GlobalConstant.CASE_ERP_STOR_RECCCEIPT + "/caseErpStoreReceipt")
@Api(value = GlobalConstant.CASE_ERP_STOR_RECCCEIPT + "/caseErpStoreReceipt", tags = "出入库单据信息")
@AllArgsConstructor
public class CaseErpStoreReceiptController {

    private ICaseErpStoreReceiptService caseErpStoreReceiptService;

    private ICaseErpStoreReceiptDetailService caseErpStoreReceiptDetailService;

    @GetMapping(value = "/page")
    @ApiOperation("出入库单据信息列表分页")
    public R page(CaseErpStoreReceiptPageDto dto) {
        LambdaQueryWrapper<CaseErpStoreReceipt> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper
                .like(StrUtil.isNotBlank(dto.getKeyword()), CaseErpStoreReceipt::getCode,dto.getKeyword())
                .eq(ObjectUtil.isNotEmpty(dto.getType()),CaseErpStoreReceipt::getType,dto.getType())
                .or()
                .like(StrUtil.isNotBlank(dto.getKeyword()), CaseErpStoreReceipt::getName,dto.getKeyword())
                .eq(ObjectUtil.isNotEmpty(dto.getType()),CaseErpStoreReceipt::getType,dto.getType())
                .orderByDesc(CaseErpStoreReceipt::getCreateDate)
                .select(CaseErpStoreReceipt.class, x -> VoToColumnUtil.fieldsToColumns(CaseErpStoreReceiptPageVo.class).contains(x.getProperty()));
        IPage<CaseErpStoreReceipt> page = caseErpStoreReceiptService.page(ConventPage.getPage(dto), queryWrapper);
        PageOutput<CaseErpStoreReceiptPageVo> pageOutput = ConventPage.getPageOutput(page, CaseErpStoreReceiptPageVo.class);
        return R.ok(pageOutput);
    }

    @PostMapping
    @ApiOperation(value = "新增出入库单据信息")
    @Transactional
    public R add(@RequestBody AddCaseErpStoreReceiptDto dto) {
        return R.ok(caseErpStoreReceiptService.add(dto));
    }


    @PutMapping
    @ApiOperation(value = "修改出入库单据信息")
    @Transactional
    public R update(@Valid @RequestBody UpdateCaseErpStoreReceiptDto dto) {
        return R.ok(caseErpStoreReceiptService.edit(dto));
    }


    @GetMapping(value = "/info")
    @ApiOperation(value = "根据id查询出入库单据信息")
    public R info(@RequestParam Long id) {
        CaseErpStoreReceipt caseErpStoreReceipt = caseErpStoreReceiptService.getById(id);
        if (caseErpStoreReceipt == null) {
            R.error("找不到此出入库单据信息");
        }
        CaseErpStoreReceiptInfoVo caseErpStoreReceiptInfoVo = BeanUtil.toBean(caseErpStoreReceipt, CaseErpStoreReceiptInfoVo.class);
        List<CaseErpStoreReceiptDetail> list = caseErpStoreReceiptDetailService.list(Wrappers.<CaseErpStoreReceiptDetail>query().lambda().eq(CaseErpStoreReceiptDetail::getStoreReceiptId, caseErpStoreReceipt.getId()));
        caseErpStoreReceiptInfoVo.setCaseErpStoreReceiptDetailList(list);
        return R.ok(caseErpStoreReceiptInfoVo);
    }


    @DeleteMapping
    @ApiOperation(value = "删除")
    @XjrLog(value = "删除出入库单据信息")
    public R delete(@Valid @RequestBody Long id) {
        return R.ok(caseErpStoreReceiptService.delete(id));
    }
}
