package com.xjrsoft.module.erpModel.caseErpStore.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.lang.TypeReference;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.xjrsoft.common.constant.GlobalConstant;
import com.xjrsoft.common.enums.YesOrNoEnum;
import com.xjrsoft.common.utils.RedisUtil;
import com.xjrsoft.module.erpModel.caseErpMaterial.entity.CaseErpMaterial;
import com.xjrsoft.module.erpModel.caseErpMaterial.service.ICaseErpMaterialService;
import com.xjrsoft.module.erpModel.caseErpStore.dto.AddCaseErpStoreReceiptDto;
import com.xjrsoft.module.erpModel.caseErpStore.dto.UpdateCaseErpStoreReceiptDto;
import com.xjrsoft.module.erpModel.caseErpStore.entity.CaseErpStoreReceipt;
import com.xjrsoft.module.erpModel.caseErpStore.entity.CaseErpStoreReceiptDetail;
import com.xjrsoft.module.erpModel.caseErpStore.mapper.CaseErpStoreReceiptMapper;
import com.xjrsoft.module.erpModel.caseErpStore.service.ICaseErpStoreReceiptDetailService;
import com.xjrsoft.module.erpModel.caseErpStore.service.ICaseErpStoreReceiptService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xjrsoft.module.organization.entity.User;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 出入库单据信息表【case_erp_store_receipt】 服务实现类
 * </p>
 *
 * @author hnyyzy
 * @since 2023-07-20
 */
@Service
@AllArgsConstructor
public class CaseErpStoreReceiptServiceImpl extends ServiceImpl<CaseErpStoreReceiptMapper, CaseErpStoreReceipt> implements ICaseErpStoreReceiptService {

    private final ICaseErpMaterialService caseErpMaterialService;

    private final ICaseErpStoreReceiptDetailService caseErpStoreReceiptDetailService;

    private final RedisUtil redisUtil;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean delete(Long id) {
        this.removeById(id);
        this.deleteDetail(id);
        return true;
    }

    /**
     * 删除出入库表详情数据以及把库存还原
     * @param id 出入库表主键id
     */
    public void deleteDetail(Long id){
        List<CaseErpMaterial> caseErpMaterialList = caseErpMaterialService.list();
        List<CaseErpMaterial> updateCaseErpMaterialList = new ArrayList<>();
        //删除之前需要先把入库或出库的库存给还原回来
        List<CaseErpStoreReceiptDetail> list = caseErpStoreReceiptDetailService.list(Wrappers.<CaseErpStoreReceiptDetail>query().lambda().eq(CaseErpStoreReceiptDetail::getStoreReceiptId, id));
        for (CaseErpStoreReceiptDetail caseErpStoreReceiptDetail : list) {
            if (StrUtil.isNotBlank(caseErpStoreReceiptDetail.getCode())){
                CaseErpMaterial caseErpMaterial = caseErpMaterialList.stream().filter(x -> x.getCode().equals(caseErpStoreReceiptDetail.getCode())).findFirst().orElse(new CaseErpMaterial());
                if (ObjectUtil.isNotEmpty(caseErpMaterial.getInventory())){
                    if (caseErpStoreReceiptDetail.getOperateType() == 2){//销售订单
                        caseErpMaterial.setInventory(caseErpMaterial.getInventory().add(caseErpStoreReceiptDetail.getCount()));
                        updateCaseErpMaterialList.add(caseErpMaterial);
                    }else if (caseErpStoreReceiptDetail.getOperateType() == 3){//采购订单
                        caseErpMaterial.setInventory(caseErpMaterial.getInventory().subtract(caseErpStoreReceiptDetail.getCount()));
                        updateCaseErpMaterialList.add(caseErpMaterial);
                    }
                }
            }
        }
        caseErpMaterialService.updateBatchById(updateCaseErpMaterialList);
        caseErpStoreReceiptDetailService.remove(Wrappers.<CaseErpStoreReceiptDetail>query().lambda().eq(CaseErpStoreReceiptDetail::getStoreReceiptId, id));
    }

    /**
     * 添加出入库表详情数据以及把库存还原
     * @param caseErpStoreReceiptDetailList
     * @param caseErpStoreReceipt
     */
    public void addDetail(List<CaseErpStoreReceiptDetail> caseErpStoreReceiptDetailList,CaseErpStoreReceipt caseErpStoreReceipt){
        List<CaseErpMaterial> caseErpMaterialList = caseErpMaterialService.list();
        List<CaseErpMaterial> updateCaseErpMaterialList = new ArrayList<>();
        for (CaseErpStoreReceiptDetail caseErpStoreReceiptDetail : caseErpStoreReceiptDetailList) {
            caseErpStoreReceiptDetail.setStoreReceiptId(caseErpStoreReceipt.getId());
            if (caseErpStoreReceipt.getType() == YesOrNoEnum.YES.getCode()) { // 入库则是采购订单
                caseErpStoreReceiptDetail.setOperateType(3);//设置操作类型为采购订单
                if (StrUtil.isNotBlank(caseErpStoreReceiptDetail.getCode())){
                    CaseErpMaterial caseErpMaterial = caseErpMaterialList.stream().filter(x -> x.getCode().equals(caseErpStoreReceiptDetail.getCode())).findFirst().orElse(new CaseErpMaterial());
                    if (ObjectUtil.isNotEmpty(caseErpMaterial.getInventory())){//库存数不为空，采购订单则需增加库存数量,物料表库存数量需要增加
                        caseErpStoreReceiptDetail.setLocalInventory(caseErpMaterial.getInventory().add(caseErpStoreReceiptDetail.getCount()));//设置当前库存数量
                        caseErpMaterial.setInventory(caseErpMaterial.getInventory());
                        updateCaseErpMaterialList.add(caseErpMaterial);
                    }
                }
            }else {//出库则是销售订单
                caseErpStoreReceiptDetail.setOperateType(2);//设置操作类型为销售订单
                if (StrUtil.isNotBlank(caseErpStoreReceiptDetail.getCode())){
                    CaseErpMaterial caseErpMaterial = caseErpMaterialList.stream().filter(x -> x.getCode().equals(caseErpStoreReceiptDetail.getCode())).findFirst().orElse(new CaseErpMaterial());
                    if (ObjectUtil.isNotEmpty(caseErpMaterial.getInventory())){//库存数不为空，销售则需减少库存数量,物料表库存数量需要减少
                        caseErpStoreReceiptDetail.setLocalInventory(caseErpMaterial.getInventory().subtract(caseErpStoreReceiptDetail.getCount()));//设置当前库存数量
                        caseErpMaterial.setInventory(caseErpMaterial.getInventory());
                        updateCaseErpMaterialList.add(caseErpMaterial);
                    }
                }
            }
        }
        caseErpStoreReceiptDetailService.saveBatch(caseErpStoreReceiptDetailList);
        caseErpMaterialService.updateBatchById(updateCaseErpMaterialList);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean add(AddCaseErpStoreReceiptDto dto) {
        CaseErpStoreReceipt caseErpStoreReceipt = BeanUtil.toBean(dto, CaseErpStoreReceipt.class);
        if (StrUtil.isNotBlank(caseErpStoreReceipt.getPersonIds())) {
            List<User> allUser = redisUtil.get(GlobalConstant.USER_CACHE_KEY, new TypeReference<List<User>>() {
            });
            String allUserIdStr = StrUtil.join(StringPool.COMMA, caseErpStoreReceipt.getPersonIds());
            List<Long> ids = Arrays.stream(allUserIdStr.split(StringPool.COMMA)).map(Convert::toLong).collect(Collectors.toList());
            List<String> names = allUser.stream().filter(x -> ids.contains(x.getId())).map(User::getName).collect(Collectors.toList());
            caseErpStoreReceipt.setPersonNames(StrUtil.join(StringPool.COMMA, names));
        }
        this.save(caseErpStoreReceipt);
        List<CaseErpStoreReceiptDetail> caseErpStoreReceiptDetailList = BeanUtil.copyToList(dto.getAddCaseErpStoreReceiptDetailDtoList(), CaseErpStoreReceiptDetail.class);
        if (CollectionUtil.isNotEmpty(caseErpStoreReceiptDetailList)) {
            this.addDetail(caseErpStoreReceiptDetailList,caseErpStoreReceipt);
        }
        return true;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean edit(UpdateCaseErpStoreReceiptDto dto) {
        CaseErpStoreReceipt caseErpStoreReceipt = BeanUtil.toBean(dto, CaseErpStoreReceipt.class);
        if (StrUtil.isNotBlank(caseErpStoreReceipt.getPersonIds())) {
            List<User> allUser = redisUtil.get(GlobalConstant.USER_CACHE_KEY, new TypeReference<List<User>>() {
            });
            String allUserIdStr = StrUtil.join(StringPool.COMMA, caseErpStoreReceipt.getPersonIds());
            List<Long> ids = Arrays.stream(allUserIdStr.split(StringPool.COMMA)).map(Convert::toLong).collect(Collectors.toList());
            List<String> names = allUser.stream().filter(x -> ids.contains(x.getId())).map(User::getName).collect(Collectors.toList());
            caseErpStoreReceipt.setPersonNames(StrUtil.join(StringPool.COMMA, names));
        }
        this.updateById(caseErpStoreReceipt);
        if (CollectionUtil.isNotEmpty(dto.getAddCaseErpStoreReceiptDetailDtoList())) { // 先删除详情再新增
            this.deleteDetail(caseErpStoreReceipt.getId());
            List<CaseErpStoreReceiptDetail> caseErpStoreReceiptDetailList = BeanUtil.copyToList(dto.getAddCaseErpStoreReceiptDetailDtoList(), CaseErpStoreReceiptDetail.class);
            if (CollectionUtil.isNotEmpty(caseErpStoreReceiptDetailList)) {
                this.addDetail(caseErpStoreReceiptDetailList,caseErpStoreReceipt);
            }
        } else {
            this.deleteDetail(caseErpStoreReceipt.getId());
        }
        return true;
    }


}
