package com.xjrsoft.module.generator.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.io.IoUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.db.Db;
import cn.hutool.db.DbUtil;
import cn.hutool.db.meta.Column;
import cn.hutool.db.meta.JdbcType;
import cn.hutool.db.meta.MetaUtil;
import cn.hutool.db.meta.Table;
import cn.hutool.extra.pinyin.PinyinUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.annotation.DbType;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.toolkit.JdbcUtils;
import com.xjrsoft.common.constant.GlobalConstant;
import com.xjrsoft.common.enums.FormTemplateType;
import com.xjrsoft.common.enums.MenuType;
import com.xjrsoft.common.enums.TsColumnType;
import com.xjrsoft.common.enums.YesOrNoEnum;
import com.xjrsoft.common.exception.MyException;
import com.xjrsoft.common.model.datasource.MyColumnInfo;
import com.xjrsoft.common.model.datasource.MyTableInfo;
import com.xjrsoft.common.model.generator.ComponentConfig;
import com.xjrsoft.common.utils.DatasourceUtil;
import com.xjrsoft.common.utils.JdbcToJavaUtil;
import com.xjrsoft.common.utils.LocalDateTimeUtil;
import com.xjrsoft.config.GeneratePathConfig;
import com.xjrsoft.module.authority.service.IDataAuthTableRelationService;
import com.xjrsoft.module.form.entity.FormDesignConfig;
import com.xjrsoft.module.form.entity.FormTemplate;
import com.xjrsoft.module.form.service.IFormTemplateService;
import com.xjrsoft.module.form.utils.FromTemplateUtil;
import com.xjrsoft.module.generator.constant.ComponentTypeConstant;
import com.xjrsoft.module.generator.constant.EntityConstant;
import com.xjrsoft.module.generator.dto.*;
import com.xjrsoft.module.generator.entity.*;
import com.xjrsoft.module.generator.service.IGeneratorService;
import com.xjrsoft.module.generator.utils.GeneratorUtil;
import com.xjrsoft.module.generator.utils.SqlUtil;
import com.xjrsoft.module.generator.vo.ColumnInfoVo;
import com.xjrsoft.module.generator.vo.GeneratorCodeVo;
import com.xjrsoft.module.generator.vo.TableInfoVo;
import com.xjrsoft.module.system.entity.*;
import com.xjrsoft.module.system.mapper.MenuMapper;
import com.xjrsoft.module.system.service.*;
import freemarker.template.Configuration;
import freemarker.template.Template;
import lombok.AllArgsConstructor;
import lombok.SneakyThrows;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.BooleanUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.ui.freemarker.FreeMarkerTemplateUtils;
import org.springframework.web.servlet.view.freemarker.FreeMarkerConfigurer;

import javax.sql.DataSource;
import javax.validation.constraints.NotNull;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.nio.charset.StandardCharsets;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @title: GeneratorServiceImpl
 * @Author tzx
 * @Date: 2022/4/16 23:16
 * @Version 1.0
 */
@Service
@AllArgsConstructor
public class GeneratorServiceImpl implements IGeneratorService {

    private static final String TEMPLATE_PATH = "src/main/resources/template";

    private static final String ENTITY_TEMPLATE_NAME = "entity.java.ftl";

    private static final String ADD_DTO_TEMPLATE_NAME = "add.dto.java.ftl";

    private static final String UPDATE_DTO_TEMPLATE_NAME = "update.dto.java.ftl";

    private static final String PAGE_DTO_TEMPLATE_NAME = "page.list.dto.java.ftl";

    private static final String PAGE_VO_TEMPLATE_NAME = "page.list.vo.java.ftl";

    private static final String INFO_VO_TEMPLATE_NAME = "vo.java.ftl";

    private static final String MAPPPER_TEMPLATE_NAME = "mapper.java.ftl";

    private static final String SERVICE_TEMPLATE_NAME = "service.java.ftl";

    private static final String SERVICE_IMPL_TEMPLATE_NAME = "service.impl.java.ftl";

    private static final String CONTROLLER_TEMPLATE_NAME = "controller.java.ftl";

    private static final List<Class> EQUALS_TYPE_CLASS = Arrays.asList(Integer.class, Long.class, Double.class, Float.class, Boolean.class);

    private final MenuMapper menuMapper;

    private final IMenuButtonService menuButtonService;

    private final IMenuColumnService menuColumnService;

    private final IMenuFormService menuFormService;

    private final GeneratePathConfig generatePathConfig;

    private final IFormTemplateService formTemplateService;

    private final IDataAuthTableRelationService dataAuthTableRelationService;

    private final ICodeSchemaService codeSchemaService;

    private final FreeMarkerConfigurer freeMarkerConfigurer;

    private final IDatabaselinkService databaselinkService;


    @Override
    @SneakyThrows
    public GeneratorCodeVo getPreviewCodes(DataFirstPreviewDto dto) {

        GeneratorConfig generatorConfig = BeanUtil.toBean(dto, GeneratorConfig.class);

        Optional<TableConfig> tableConfigOptional = dto.getTableConfigs().stream().filter(TableConfig::getIsMain).findFirst();
        //主表
        TableConfig mainTable;
        if (tableConfigOptional.isPresent()) {
            mainTable = tableConfigOptional.get();
        } else {
            throw new MyException("请选择主表");
        }
        List<Table> tableInfos = getTableInfos(generatorConfig);

        return getGeneratorCodeVo(generatorConfig, mainTable, tableInfos);
    }


    @Override
    @SneakyThrows
    @Transactional(rollbackFor = Exception.class)
    public Boolean generateCodes(DataFirstGeneratorDto dto) {
        GeneratorConfig generatorConfig = BeanUtil.toBean(dto, GeneratorConfig.class);

        Optional<TableConfig> tableConfigOptional = dto.getTableConfigs().stream().filter(TableConfig::getIsMain).findFirst();
        //主表
        TableConfig mainTable;
        if (tableConfigOptional.isPresent()) {
            mainTable = tableConfigOptional.get();
        } else {
            throw new MyException("请选择主表");
        }

        OutputConfig outputConfig = dto.getOutputConfig();
        String databaseId = dto.getDatabaseId();
        if (BooleanUtils.isTrue(outputConfig.getIsDataAuth())) {
            // 添加权限字段 rule_user_id
            DataSource dataSource = DatasourceUtil.getDataSource(databaseId);
            String[] columnNames = MetaUtil.getColumnNames(dataSource, mainTable.getTableName());
            if (!ArrayUtils.contains(columnNames, GlobalConstant.AUTH_USER_ID)) {
                DbType dbType = databaselinkService.getDbType(databaseId);
                Db.use(dataSource).execute(SqlUtil.buildAddDataAuthFieldSql(dbType, mainTable.getTableName()));
            }
        }

        //修改 getTableInfos  也记得修改 FromTemplateServiceImpl 的同名方法
        List<Table> tableInfos = getTableInfos(generatorConfig);

        createCodeFile(generatorConfig, mainTable, tableInfos, dto.getFrontCode());


        //生成代码的同时 新增代码模板数据
        FormTemplate template = new FormTemplate();
        template.setFormType(FormTemplateType.SYSTEM.getCode());
        template.setCategory(dto.getCategoryId());
        template.setName(StrUtil.isNotBlank(outputConfig.getComment()) ? outputConfig.getComment() : dto.getName());

        FormDesignConfig formDesignConfig = new FormDesignConfig();
        formDesignConfig.setDatabaseId(databaseId);
        formDesignConfig.setTableConfigs(dto.getTableConfigs());
        formDesignConfig.setFormEventConfig(dto.getFormEventConfig());

        List<TableStructureConfig> tableStructureConfigs = FromTemplateUtil.wrapperTableStructureConfig(dto.getFormJson().getList(), mainTable);
        formDesignConfig.setTableStructureConfigs(tableStructureConfigs);
        formDesignConfig.setFormJson(dto.getFormJson());

        template.setFormJson(JSONUtil.toJsonStr(formDesignConfig));


        //保存菜单、权限数据、
        if (generatorConfig.getOutputConfig().getIsMenu()) {
            Menu menu = insertMenuConfig(generatorConfig);
            template.setId(menu.getFormId());
        }
        // 保存表单设计需要在保存菜单之后
        formTemplateService.saveOrUpdate(template);
        // 保存数据权限关系
        if (BooleanUtils.isTrue(outputConfig.getIsDataAuth())) {
            dataAuthTableRelationService.saveDataAuthTableRelations(mainTable.getTableName(), outputConfig.getDataAuthList());
        }
        // 保存代码模板
        saveCodeSchema(dto.getId(), outputConfig, JSONUtil.toJsonStr(dto));
        return true;
    }


    @Override
    @SneakyThrows
    public GeneratorCodeVo getCodeFirstPreviewCodes(CodeFirstPreviewDto dto) {

        GeneratorConfig generatorConfig = BeanUtil.toBean(dto, GeneratorConfig.class);

        // 默认将表结构 表名  字段名 改为全小写 蛇形 例如  XjrParent -> xjr_parent   XjrChild -> xjr_child
        // 全转未小写后 如果是oracle、达梦 数据库 也会自动变成全大写 无需关心
        for (TableStructureConfig tableStructureConfig : generatorConfig.getTableStructureConfigs()) {
            tableStructureConfig.setTableName(StrUtil.toUnderlineCase(tableStructureConfig.getTableName()));
            for (TableFieldConfig tableFieldConfig : tableStructureConfig.getTableFieldConfigs()) {
                tableFieldConfig.setFieldName(StrUtil.toUnderlineCase(tableFieldConfig.getFieldName()));
            }
        }


        Optional<TableStructureConfig> mainTableOp = dto.getTableStructureConfigs().stream().filter(TableStructureConfig::getIsMain).findFirst();
        //主表
        TableConfig mainTable = new TableConfig();
        if (mainTableOp.isPresent()) {
            mainTable.setIsMain(true);
            mainTable.setTableName(mainTableOp.get().getTableName());
            mainTable.setPkField(GlobalConstant.DEFAULT_PK);
        } else {
            throw new MyException("主表不存在");
        }

        //创建数据库表。  子表默认多创建一个 主表id 的字段 用于关联
        createTable(generatorConfig);

        List<Table> tableInfos = new ArrayList<>();
        // 默认拼接主表
        generatorConfig.setTableConfigs(new ArrayList<>());
        for (TableStructureConfig tableStructureConfig : dto.getTableStructureConfigs()) {
            //判断是否为默认数据源
            DataSource dataSource = null;
            if (StrUtil.equalsIgnoreCase(dto.getDatabaseId(), GlobalConstant.DEFAULT_DATASOURCE_KEY)) {
                dataSource = DatasourceUtil.getDatasourceMaster();
            } else {
                dataSource = DatasourceUtil.getDataSource(dto.getDatabaseId());

            }
            tableInfos.add(MetaUtil.getTableMeta(dataSource, tableStructureConfig.getTableName()));

            Integer operator = tableStructureConfig.getOperator();
            if (operator == null || operator == 2 || operator == 4) {
                //因为是预览代码 并不一定会生成功能 所以新创建的表必须得删掉。
                String dropSql = DatasourceUtil.wrapperDropSql(tableStructureConfig.getTableName());
                DbUtil.use(dataSource).execute(dropSql);
            }

            TableConfig tableConfig = new TableConfig();
            tableConfig.setTableName(tableStructureConfig.getTableName());
            tableConfig.setIsMain(tableStructureConfig.getIsMain());

            //子表 的关联字段默认使用 parent_id
            if (!tableStructureConfig.getIsMain()) {
                tableConfig.setRelationField(GlobalConstant.DEFAULT_FK);
                tableConfig.setRelationTableField(GlobalConstant.DEFAULT_PK);
            }

            tableConfig.setPkField(GlobalConstant.DEFAULT_PK);
            tableConfig.setPkType(GlobalConstant.DEFAULT_PK_TYPE);

            generatorConfig.getTableConfigs().add(tableConfig);


        }
        return getGeneratorCodeVo(generatorConfig, mainTable, tableInfos);
    }


    @Override
    @SneakyThrows
    @Transactional(rollbackFor = Exception.class)
    public Boolean generateCodeFirstCodes(CodeFirstGeneratorDto dto) {
        GeneratorConfig generatorConfig = BeanUtil.toBean(dto, GeneratorConfig.class);
        // 默认将表结构 表名  字段名 改为全小写 蛇形 例如  XjrParent -> xjr_parent   XjrChild -> xjr_child
        // 全转未小写后 如果是oracle、达梦 数据库 也会自动变成全大写 无需关心
        for (TableStructureConfig tableStructureConfig : generatorConfig.getTableStructureConfigs()) {
            tableStructureConfig.setTableName(StrUtil.toUnderlineCase(tableStructureConfig.getTableName()));
            for (TableFieldConfig tableFieldConfig : tableStructureConfig.getTableFieldConfigs()) {
                tableFieldConfig.setFieldName(StrUtil.toUnderlineCase(tableFieldConfig.getFieldName()));
            }
        }

        Optional<TableStructureConfig> mainTableOp = dto.getTableStructureConfigs().stream().filter(TableStructureConfig::getIsMain).findFirst();
        //主表
        TableConfig mainTable = new TableConfig();
        if (mainTableOp.isPresent()) {
            mainTable.setIsMain(true);
            mainTable.setTableName(mainTableOp.get().getTableName());
            mainTable.setPkField(GlobalConstant.DEFAULT_PK);
        } else {
            throw new MyException("主表不存在");
        }

        createTable(generatorConfig);

        List<Table> tableInfos = new ArrayList<>();

        // 默认拼接主表
        generatorConfig.setTableConfigs(new ArrayList<>());
        for (TableStructureConfig tableStructureConfig : generatorConfig.getTableStructureConfigs()) {
            //判断是否为默认数据源
            if (StrUtil.equalsIgnoreCase(dto.getDatabaseId(), GlobalConstant.DEFAULT_DATASOURCE_KEY)) {
                DataSource datasourceMaster = DatasourceUtil.getDatasourceMaster();
                tableInfos.add(MetaUtil.getTableMeta(datasourceMaster, tableStructureConfig.getTableName()));
            } else {
                DataSource dataSource = DatasourceUtil.getDataSource(dto.getDatabaseId());
                tableInfos.add(MetaUtil.getTableMeta(dataSource, tableStructureConfig.getTableName()));
            }
            TableConfig tableConfig = new TableConfig();
            tableConfig.setTableName(tableStructureConfig.getTableName());
            tableConfig.setIsMain(tableStructureConfig.getIsMain());

            //子表 的关联字段默认使用 parent_id
            if (!tableStructureConfig.getIsMain()) {
                tableConfig.setRelationField(GlobalConstant.DEFAULT_FK);
                tableConfig.setRelationTableField(GlobalConstant.DEFAULT_PK);
            }

            tableConfig.setPkField(GlobalConstant.DEFAULT_PK);
            tableConfig.setPkType(GlobalConstant.DEFAULT_PK_TYPE);

            generatorConfig.getTableConfigs().add(tableConfig);


        }

        createCodeFile(generatorConfig, mainTable, tableInfos, dto.getFrontCode());

        //生成代码的同时 新增代码模板数据
        FormTemplate template = new FormTemplate();
        template.setFormType(FormTemplateType.SYSTEM.getCode());
        template.setCategory(dto.getCategoryId());
        OutputConfig outputConfig = dto.getOutputConfig();
        template.setName(StrUtil.isNotBlank(outputConfig.getComment()) ? outputConfig.getComment() : dto.getName());

        FormDesignConfig formDesignConfig = new FormDesignConfig();
        formDesignConfig.setDatabaseId(dto.getDatabaseId());

        List<TableConfig> tableConfigs = FromTemplateUtil.wrapperTableConfig(dto.getTableStructureConfigs());

        formDesignConfig.setTableConfigs(tableConfigs);
        formDesignConfig.setTableStructureConfigs(dto.getTableStructureConfigs());
        formDesignConfig.setFormJson(dto.getFormJson());
        formDesignConfig.setFormEventConfig(dto.getFormEventConfig());
        template.setFormJson(JSONUtil.toJsonStr(formDesignConfig));


        //保存菜单、权限数据
        if (generatorConfig.getOutputConfig().getIsMenu()) {
            Menu menu = insertMenuConfig(generatorConfig);
            template.setId(menu.getFormId());
        }
        // 保存表单设计需要在保存菜单之后
        formTemplateService.saveOrUpdate(template);
        // 保存数据权限关系
        if (BooleanUtils.isTrue(outputConfig.getIsDataAuth())) {
            dataAuthTableRelationService.saveDataAuthTableRelations(mainTable.getTableName(), outputConfig.getDataAuthList());
        }
        // 保存代码模板
        saveCodeSchema(dto.getId(), outputConfig, JSONUtil.toJsonStr(dto));
        return true;
    }

    @Override
    public List<TableInfoVo> getTableInfo(TableInfoDto dto) {
        return getTableInfoVos(dto);
    }

    @Override
    @SneakyThrows
    public Boolean generateAppCodes(GeneratorAppDto dto) {

        FrontCode frontCode = new FrontCode();
        frontCode.setApiCode(dto.getApiCode());
        frontCode.setListCode(dto.getListCode());
        frontCode.setFormCode(dto.getFormCode());
        frontCode.setConfigJsonCode(dto.getConfigJsonCode());
        createAppCodeFile(frontCode, dto.getClassName(), dto.getOutputArea());
        return Boolean.TRUE;
    }


    /**
     * 根据配置生成文件
     *
     * @param generatorConfig
     * @param mainTable
     * @param tableInfos
     * @throws FileNotFoundException
     */
    private void createCodeFile(GeneratorConfig generatorConfig, TableConfig mainTable, List<Table> tableInfos, FrontCode frontCode) throws FileNotFoundException {
        String className = generatorConfig.getOutputConfig().getClassName();
        String dirValue = generatorConfig.getOutputConfig().getOutputValue();

        //---------------------------------------生成entity开始----------------------------------------------------
        {
            Map<String, String> entityCodeMap = getEntityCode(generatorConfig, tableInfos);

            for (Map.Entry<String, String> entry : entityCodeMap.entrySet()) {
                writeFile(getEntityOutputDir(dirValue), entry.getKey() + StringPool.DOT_JAVA, entry.getValue());
            }
        }
        //---------------------------------------生成entity结束----------------------------------------------------


        //********************************************生成dto开始************************************************
        {
            Map<String, String> dtoMap = new HashMap<>(3);
            dtoMap.putAll(getAddDtoCode(generatorConfig, tableInfos));
            dtoMap.putAll(getUpdateDtoCode(generatorConfig, tableInfos));
            dtoMap.putAll(getPageDtoCode(generatorConfig, tableInfos, mainTable));

            for (Map.Entry<String, String> entry : dtoMap.entrySet()) {
                writeFile(getDtoOutputDir(dirValue), entry.getKey() + StringPool.DOT_JAVA, entry.getValue());
            }
        }
        //********************************************生成dto结束************************************************

        //###############################################生成Vo开始###############################################
        {
            Map<String, String> voMap = new HashMap<>(2);
            voMap.putAll(getPageVoCode(generatorConfig, tableInfos, mainTable));
            voMap.putAll(getInfoVoCode(generatorConfig, tableInfos, mainTable));

            for (Map.Entry<String, String> entry : voMap.entrySet()) {
                writeFile(getVoOutputDir(dirValue), entry.getKey() + StringPool.DOT_JAVA, entry.getValue());
            }
        }
        //###############################################生成Vo结束###############################################


        //---------------------------------------生成三层代码开始----------------------------------------------------
        {
            Map<String, String> mapperCode = getMapperCode(generatorConfig, tableInfos);

            for (Map.Entry<String, String> entry : mapperCode.entrySet()) {
                writeFile(getMapperOutputDir(dirValue), entry.getKey() + StringPool.DOT_JAVA, entry.getValue());
            }
        }

        {
            Map<String, String> serviceCode = getServiceCode(generatorConfig, mainTable, tableInfos.size() > 1);

            for (Map.Entry<String, String> entry : serviceCode.entrySet()) {
                writeFile(getServiceOutputDir(dirValue), entry.getKey() + StringPool.DOT_JAVA, entry.getValue());
            }
        }

        {
            Map<String, String> serviceImplCode = getServiceImplCode(generatorConfig, mainTable, tableInfos.size() > 1);
            for (Map.Entry<String, String> entry : serviceImplCode.entrySet()) {
                writeFile(getServiceImplOutputDir(dirValue), entry.getKey() + StringPool.DOT_JAVA, entry.getValue());
            }
        }

        {
            Map<String, String> controllerCode = getControllerCode(generatorConfig, tableInfos, mainTable);
            for (Map.Entry<String, String> entry : controllerCode.entrySet()) {
                writeFile(getControllerOutputDir(dirValue), entry.getKey() + StringPool.DOT_JAVA, entry.getValue());
            }
        }
        //---------------------------------------生成三层代码结束----------------------------------------------------


        //---------------------------------------生成页面代码开始----------------------------------------------------
        {
            String listFileName = "index.vue";
            writeFile(getListOutputDir(dirValue, className), listFileName, frontCode.getListCode());
        }
        //---------------------------------------生成页面代码结束----------------------------------------------------

        //---------------------------------------生成表单代码开始----------------------------------------------------
        {
            String suffix = StrUtil.upperFirst(generatorConfig.getFormJson().getConfig().getFormType());
            String formFileName = StrUtil.upperFirst(StrUtil.toCamelCase(className)) + suffix + ".vue";
            writeFile(getFormOutputDir(dirValue, className), formFileName, frontCode.getFormCode());
        }
        //---------------------------------------生成表单代码开始----------------------------------------------------

        //---------------------------------------生成表单、列表 配置 json开始----------------------------------------------------
        {
            String configFileName = "config.ts";
            writeFile(getFormOutputDir(dirValue, className), configFileName, frontCode.getConfigJsonCode());
        }
        //---------------------------------------生成表单、列表 配置 json开始----------------------------------------------------


        //---------------------------------------生成前端请求文件代码开始----------------------------------------------------
        {
            String apiFileName = "index.ts";
            writeFile(getApiOutputDir(dirValue, className), apiFileName, frontCode.getApiCode());
        }
        //---------------------------------------生成前端请求文件代码开始----------------------------------------------------


        //---------------------------------------生成前端模型代码开始----------------------------------------------------
        {
            String modelFileName = StrUtil.upperFirst(StrUtil.toCamelCase(className)) + "Model.ts";
            writeFile(getModelOutputDir(dirValue, className), modelFileName, frontCode.getModelCode());
        }
        //---------------------------------------生成前端模型代码开始----------------------------------------------------

    }


    /**
     * 根据前端传入参数 生成APP代码
     *
     * @param frontCode 前端代码
     * @param className 功能类名
     * @param dirValue  输出区域
     * @throws FileNotFoundException
     */
    private void createAppCodeFile(FrontCode frontCode, String className, String dirValue) throws FileNotFoundException {


        //---------------------------------------生成页面代码开始----------------------------------------------------
        {
            String listFileName = "list.vue";
            writeFile(getAppListOutputDir(dirValue, className), listFileName, frontCode.getListCode());
        }
        //---------------------------------------生成页面代码结束----------------------------------------------------

        //---------------------------------------生成表单代码开始----------------------------------------------------
        {
            String formFileName = "form.vue";
            writeFile(getAppFormOutputDir(dirValue, className), formFileName, frontCode.getFormCode());
        }
        //---------------------------------------生成表单代码开始----------------------------------------------------

        //---------------------------------------生成表单、列表 配置 json开始----------------------------------------------------
        {
            String configFileName = "index.js";
            writeFile(getAppConfigOutputDir(dirValue, className), configFileName, frontCode.getConfigJsonCode());
        }
        //---------------------------------------生成表单、列表 配置 json开始----------------------------------------------------


        //---------------------------------------生成前端请求文件代码开始----------------------------------------------------
        {
            String apiFileName = "index.js";
            writeFile(getAppApiOutputDir(dirValue, className), apiFileName, frontCode.getApiCode());
        }
        //---------------------------------------生成前端请求文件代码开始----------------------------------------------------


    }

    /**
     * 根据配置预览代码
     *
     * @param generatorConfig
     * @param mainTable
     * @param tableInfos
     * @return
     */
    private GeneratorCodeVo getGeneratorCodeVo(GeneratorConfig generatorConfig, TableConfig mainTable, List<Table> tableInfos) {
        GeneratorCodeVo vo = new GeneratorCodeVo();

        //生成实体类
        vo.setEntityCode(getEntityCode(generatorConfig, tableInfos));


        Map<String, String> dtoMap = new HashMap<>(3);
        dtoMap.putAll(getAddDtoCode(generatorConfig, tableInfos));
        dtoMap.putAll(getUpdateDtoCode(generatorConfig, tableInfos));
        dtoMap.putAll(getPageDtoCode(generatorConfig, tableInfos, mainTable));
        vo.setDtoCode(dtoMap);


        Map<String, String> voMap = new HashMap<>(2);
        voMap.putAll(getPageVoCode(generatorConfig, tableInfos, mainTable));
        voMap.putAll(getInfoVoCode(generatorConfig, tableInfos, mainTable));
        vo.setVoCode(voMap);


        vo.setMapperCode(getMapperCode(generatorConfig, tableInfos));
        vo.setServiceCode(getServiceCode(generatorConfig, mainTable, tableInfos.size() > 1));
        vo.setImplCode(getServiceImplCode(generatorConfig, mainTable, tableInfos.size() > 1));
        vo.setControllerCode(getControllerCode(generatorConfig, tableInfos, mainTable));
        return vo;
    }

    /**
     * 创建表
     * 修改此方法逻辑，一定要对比一下 FromTemplateServiceImpl 中的 同名方法  是否 也需要修改！
     *
     * @param generatorConfig 配置
     */
    private void createTable(GeneratorConfig generatorConfig) throws SQLException {
        //------------------------------------根据配置建表开始------------------------------------------
        {
            //判断是否为默认数据源
            if (StrUtil.equalsIgnoreCase(generatorConfig.getDatabaseId(), GlobalConstant.DEFAULT_DATASOURCE_KEY)) {
                List<String> createTableSqls = DatasourceUtil.wrapperCreateTableSql(generatorConfig.getTableStructureConfigs(), DatasourceUtil.getDataSourceMasterDbType());

                for (String sql : createTableSqls) {
                    DbUtil.use(DatasourceUtil.getDatasourceMaster()).execute(sql);
                }
            } else {
                DataSource dataSource = DatasourceUtil.getDataSource(generatorConfig.getDatabaseId());
                if (dataSource == null) {
                    throw new RuntimeException("数据库连接不存在：" + generatorConfig.getDatabaseId());
                }
                DbType dbType = databaselinkService.getDbType(generatorConfig.getDatabaseId());
                List<String> createTableSqls = DatasourceUtil.wrapperCreateTableSql(generatorConfig.getTableStructureConfigs(), dbType);
                for (String sql : createTableSqls) {
                    DbUtil.use(dataSource).execute(sql);
                }
            }

        }
        //------------------------------------根据配置建表结束------------------------------------------
    }

    /**
     * 获取表结构信息
     *
     * @param generatorConfig
     * @return
     */
    private List<Table> getTableInfos(GeneratorConfig generatorConfig) {
        List<Table> tableInfos = new ArrayList<>();
        for (TableConfig tableConfig : generatorConfig.getTableConfigs()) {
            //判断是否为默认数据源
            if (StrUtil.equalsIgnoreCase(generatorConfig.getDatabaseId(), GlobalConstant.DEFAULT_DATASOURCE_KEY)) {
                tableInfos.add(MetaUtil.getTableMeta(DatasourceUtil.getDatasourceMaster(), tableConfig.getTableName()));
            } else {
                tableInfos.add(MetaUtil.getTableMeta(DatasourceUtil.getDataSource(generatorConfig.getDatabaseId()), tableConfig.getTableName()));
            }
        }
        return tableInfos;
    }

    /**
     * 获取表结构信息
     *
     * @param dto
     * @return
     */
    private List<TableInfoVo> getTableInfoVos(TableInfoDto dto) {
        List<TableInfoVo> tableInfoVos = new ArrayList<>();

        List<String> numberList = Arrays.asList("Integer", "Short", "Long", "Double", "Float", "BigDecimal", "Byte");
        String booleanStr = "Boolean";

        for (TableConfig tableConfig : dto.getTableConfigs()) {
            //判断是否为默认数据源
            DataSource dataSource = DatasourceUtil.getDataSource(dto.getDatabaseId());
            Table tableMeta = MetaUtil.getTableMeta(dataSource, tableConfig.getTableName());
            TableInfoVo vo = new TableInfoVo();
            vo.setTableName(tableMeta.getTableName());
            vo.setTableComment(tableMeta.getComment());
            vo.setColumnInfos(new ArrayList<>());
            Collection<Column> columns = tableMeta.getColumns();
            for (Column column : columns) {
                ColumnInfoVo columnInfoVo = new ColumnInfoVo();
                columnInfoVo.setColumnName(column.getName());
                columnInfoVo.setColumnLength(column.getSize());
                columnInfoVo.setIsPrimaryKey(column.isPk());
                columnInfoVo.setIsNullable(column.isNullable());

                String className = JdbcToJavaUtil.getClassName(column.getTypeEnum());

                if (numberList.contains(className)) {
                    columnInfoVo.setColumnType(TsColumnType.NUMBER.getCode());
                } else if (StrUtil.equals(className, booleanStr)) {
                    columnInfoVo.setColumnType(TsColumnType.BOOL.getCode());
                } else {
                    columnInfoVo.setColumnType(TsColumnType.STRING.getCode());
                }
                vo.getColumnInfos().add(columnInfoVo);

            }
            tableInfoVos.add(vo);

        }
        return tableInfoVos;
    }


    /**
     * 根据配置生成实体类代码
     *
     * @param generatorConfig 入参
     * @param tableInfos      表信息
     * @return 代码
     */
    @SneakyThrows
    private Map<String, String> getEntityCode(GeneratorConfig generatorConfig, List<Table> tableInfos) {
        Map<String, String> entityCodeMap = new HashMap<>(tableInfos.size());
        // 遍历数据表配置 根据表明搜索表信息  然后生成代码
        for (Table tableInfo : tableInfos) {
            //获取表的所有字段
            Collection<Column> columns = tableInfo.getColumns();
            List<FieldConfig> fieldConfigList = new ArrayList<>();

            for (Column column : columns) {
                FieldConfig fieldConfig = new FieldConfig();

                if (GlobalConstant.AUTO_INSERT.contains(column.getName())) {
                    fieldConfig.setAutoInsert(true);
                }

                if (GlobalConstant.AUTO_UPDATE.contains(column.getName())) {
                    fieldConfig.setAutoUpdate(true);
                }

                if (StrUtil.equalsIgnoreCase(column.getName(), GlobalConstant.DELETE_MARK)) {
                    fieldConfig.setDeleteMark(true);
                }
                fieldConfig.setFieldComment(column.getComment());
                fieldConfig.setPk(column.isPk());
                fieldConfig.setFieldType(JdbcToJavaUtil.getClassName(column.getTypeEnum()));
                fieldConfig.setFieldName(StrUtil.toCamelCase(column.getName()));

                fieldConfigList.add(fieldConfig);

            }


            //如果有子表  每多一个子表  多加1个长度
            Map<String, Object> entityMap = new HashMap<>(8);
            entityMap.put(EntityConstant.PACKAGE, GlobalConstant.GENERATOR_DEFAULT_PATH + StringPool.DOT + generatorConfig.getOutputConfig().getOutputValue() + StringPool.DOT + "entity");
            entityMap.put(EntityConstant.AUTH_NAME, generatorConfig.getOutputConfig().getCreator());
            entityMap.put(EntityConstant.DATE, LocalDate.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd")));
            entityMap.put(EntityConstant.TABLE_NAME, tableInfo.getTableName());
            entityMap.put(EntityConstant.TABLE_COMMENT, generatorConfig.getOutputConfig().getComment());
            //先将表明转换为驼峰命名 然后在转换为首字母大写
            entityMap.put(EntityConstant.ENTITY_CLASS_NAME, StrUtil.upperFirst(StrUtil.toCamelCase(tableInfo.getTableName())));
            entityMap.put(EntityConstant.TABLE_FIELDS, fieldConfigList);

            //如果是多表关联 则需要添加关联表的信息  判断数据表配置是否大于1  并且  是主表
            boolean isMainTable = generatorConfig.getTableConfigs().stream().anyMatch(x -> StrUtil.equals(x.getTableName(), tableInfo.getTableName()) && x.getIsMain());
            if (tableInfos.size() > 1 && isMainTable) {
                List<TableConfig> childTables = generatorConfig.getTableConfigs().stream()
                        .filter(x -> !x.getIsMain())
                        .collect(Collectors.toList());

                List<TableConfig> newChildTables = new ArrayList<>();
                for (TableConfig childTable : childTables) {
                    TableConfig newTableConfig = BeanUtil.toBean(childTable, TableConfig.class);

                    newTableConfig.setRelationField(StrUtil.toCamelCase(childTable.getRelationField()));
                    newTableConfig.setRelationTableField(StrUtil.toCamelCase(childTable.getRelationTableField()));
                    newTableConfig.setTableName(StrUtil.toCamelCase(childTable.getTableName()));

                    newChildTables.add(newTableConfig);
                }
//                Optional<TableConfig> parentTable = generatorConfig.getTableConfigs().stream().filter(TableConfig::getIsMain).findFirst();
                entityMap.put(EntityConstant.CHILD_TABLES, newChildTables);
//                parentTable.ifPresent(x -> entityMap.put(EntityConstant.PARENT_RELATION_FIELD_KEY, x.getRelationTableField()));
            }


            //生产目标代码
//            Configuration configuration = new Configuration(Configuration.DEFAULT_INCOMPATIBLE_IMPROVEMENTS);
//            configuration.setDefaultEncoding(StandardCharsets.UTF_8.name());
//            configuration.setDirectoryForTemplateLoading(new File(TEMPLATE_PATH));
//            Template template = configuration.getTemplate(ENTITY_TEMPLATE_NAME);

            Template template = freeMarkerConfigurer.getConfiguration().getTemplate(ENTITY_TEMPLATE_NAME);
            String entityContent = FreeMarkerTemplateUtils.processTemplateIntoString(template, entityMap);

            entityCodeMap.put(StrUtil.upperFirst(StrUtil.toCamelCase(tableInfo.getTableName())), entityContent);
        }

        return entityCodeMap;
    }


    /**
     * 根据配置新增dto代码
     *
     * @param generatorConfig 入参
     * @param tableInfos      表信息
     * @return 代码
     */
    @SneakyThrows
    private Map<String, String> getAddDtoCode(GeneratorConfig generatorConfig, List<Table> tableInfos) {
        Map<String, String> addCodeMap = new HashMap<>(tableInfos.size());
        Map<String, List<ComponentConfig>> tableFieldListMap = GeneratorUtil.buildFormComponentList(generatorConfig.getFormJson().getList());
        // 遍历数据表配置 根据表明搜索表信息  然后生成代码
        for (Table tableInfo : tableInfos) {
            //获取表的所有字段
            Collection<Column> columnList = tableInfo.getColumns();

            List<FieldConfig> fieldConfigList = new ArrayList<>();

            List<ComponentConfig> componentConfigList = tableFieldListMap.get(tableInfo.getTableName());
            for (Column column : columnList) {
                FieldConfig fieldConfig = new FieldConfig();

                //因为是新增Dto  不需要主键
                if (column.isPk()) {
                    continue;
                }

                //不属于 固定的审计字段 都需要生成
                if (
                        !GlobalConstant.AUTO_INSERT.contains(column.getName()) &&
                                !GlobalConstant.AUTO_UPDATE.contains(column.getName()) &&
                                !StrUtil.equalsIgnoreCase(column.getName(), GlobalConstant.DELETE_MARK) &&
                                !StrUtil.equalsIgnoreCase(column.getName(), GlobalConstant.DELETE_MARK)
                ) {
                    String fieldType = JdbcToJavaUtil.getClassName(column.getTypeEnum());
                    if (StrUtil.equalsIgnoreCase(fieldType, "LocalDateTime")) {
                        Optional<ComponentConfig> componentConfig = componentConfigList.stream().filter(config ->
                                StrUtil.equalsIgnoreCase(column.getName(), config.getBindField()) ||
                                        StrUtil.equalsIgnoreCase(column.getName(), config.getBindStartTime()) ||
                                        StrUtil.equalsIgnoreCase(column.getName(), config.getBindEndTime())
                        ).findFirst();
                        componentConfig.ifPresent(config -> {
                            String format = MapUtils.getString(config.getOptions(), "format");
                            if (StrUtil.isNotEmpty(format)) {
                                fieldConfig.setPattern(LocalDateTimeUtil.convertFontFormat(format));
                            }
                        });
                    }
                    fieldConfig.setFieldComment(column.getComment());
                    fieldConfig.setPk(false);
                    fieldConfig.setFieldType(fieldType);
                    fieldConfig.setFieldName(StrUtil.toCamelCase(column.getName()));

                    fieldConfigList.add(fieldConfig);

                }

            }


            //如果有子表  每多一个子表  多加2个长度
            Map<String, Object> entityMap = new HashMap<>(8 + (tableInfos.size() - 1) * 2);
            entityMap.put(EntityConstant.PACKAGE, GlobalConstant.GENERATOR_DEFAULT_PATH + StringPool.DOT + generatorConfig.getOutputConfig().getOutputValue() + StringPool.DOT + "dto");
            entityMap.put(EntityConstant.AUTH_NAME, generatorConfig.getOutputConfig().getCreator());
            entityMap.put(EntityConstant.DATE, LocalDate.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd")));
            entityMap.put(EntityConstant.TABLE_NAME, tableInfo.getTableName());
            entityMap.put(EntityConstant.TABLE_COMMENT, generatorConfig.getOutputConfig().getComment());
            //先将表明转换为驼峰命名 然后在转换为首字母大写
            entityMap.put(EntityConstant.ENTITY_CLASS_NAME, StrUtil.upperFirst(StrUtil.toCamelCase(tableInfo.getTableName())));
            entityMap.put(EntityConstant.TABLE_FIELDS, fieldConfigList);
            entityMap.put(EntityConstant.OUTPUT_AREA, generatorConfig.getOutputConfig().getOutputValue());

            //如果是多表关联 则需要添加关联表的信息  判断数据表配置是否大于1  并且  是主表
            boolean isMainTable = generatorConfig.getTableConfigs().stream().anyMatch(x -> StrUtil.equals(x.getTableName(), tableInfo.getTableName()) && x.getIsMain());
            if (tableInfos.size() > 1 && isMainTable) {
                List<TableConfig> childTables = generatorConfig.getTableConfigs().stream()
                        .filter(x -> !x.getIsMain())
                        .collect(Collectors.toList());
                List<TableConfig> newChildTables = new ArrayList<>();
                for (TableConfig childTable : childTables) {
                    TableConfig newTableConfig = BeanUtil.toBean(childTable, TableConfig.class);

                    newTableConfig.setRelationField(StrUtil.toCamelCase(childTable.getRelationField()));
                    newTableConfig.setRelationTableField(StrUtil.toCamelCase(childTable.getRelationTableField()));
                    newTableConfig.setTableName(StrUtil.toCamelCase(childTable.getTableName()));

                    newChildTables.add(newTableConfig);
                }
//                Optional<TableConfig> parentTable = generatorConfig.getTableConfigs().stream().filter(TableConfig::getIsMain).findFirst();
                entityMap.put(EntityConstant.CHILD_TABLES, newChildTables);
//                parentTable.ifPresent(x -> entityMap.put(EntityConstant.PARENT_RELATION_FIELD_KEY, x.getRelationTableField()));
            }


            //生产目标代码
//            Configuration configuration = new Configuration(Configuration.DEFAULT_INCOMPATIBLE_IMPROVEMENTS);
//            configuration.setDefaultEncoding(StandardCharsets.UTF_8.name());
//            configuration.setDirectoryForTemplateLoading(new File(TEMPLATE_PATH));
//            Template template = configuration.getTemplate(ADD_DTO_TEMPLATE_NAME);
            Template template = freeMarkerConfigurer.getConfiguration().getTemplate(ADD_DTO_TEMPLATE_NAME);
            String entityContent = FreeMarkerTemplateUtils.processTemplateIntoString(template, entityMap);

            addCodeMap.put("Add" + StrUtil.upperFirst(StrUtil.toCamelCase(tableInfo.getTableName())) + "Dto", entityContent);
        }

        return addCodeMap;
    }

    /**
     * 根据配置修改dto代码
     *
     * @param generatorConfig 入参
     * @param tableInfos      表信息
     * @return 代码
     */
    @SneakyThrows
    private Map<String, String> getUpdateDtoCode(GeneratorConfig generatorConfig, List<Table> tableInfos) {
        Map<String, String> updateCodeMap = new HashMap<>(tableInfos.size());
        Map<String, List<ComponentConfig>> tableFieldListMap = GeneratorUtil.buildFormComponentList(generatorConfig.getFormJson().getList());
        // 遍历数据表配置 根据表明搜索表信息  然后生成代码
        for (Table tableInfo : tableInfos) {

            //获取表的所有字段
            Collection<Column> columns = tableInfo.getColumns();
            List<FieldConfig> fieldConfigList = new ArrayList<>();

            List<ComponentConfig> componentConfigList = tableFieldListMap.get(tableInfo.getTableName());
            for (Column column : columns) {
                FieldConfig fieldConfig = new FieldConfig();

//                boolean isMainTable = generatorConfig.getTableConfigs().stream().anyMatch(x -> StrUtil.equals(x.getTableName(), tableInfo.getTableName()) && x.getIsMain());
                //子表的 UpdateDto 不需要主键
//                if (column.isPk() && !isMainTable) {
//                    continue;
//                }

                //不属于 固定的审计字段 都需要生成
                if (
                        !GlobalConstant.AUTO_INSERT.contains(column.getName()) &&
                                !GlobalConstant.AUTO_UPDATE.contains(column.getName()) &&
                                !StrUtil.equalsIgnoreCase(column.getName(), GlobalConstant.DELETE_MARK) &&
                                !StrUtil.equalsIgnoreCase(column.getName(), GlobalConstant.DELETE_MARK)
                ) {
                    String fieldType = JdbcToJavaUtil.getClassName(column.getTypeEnum());
                    if (StrUtil.equalsIgnoreCase(fieldType, "LocalDateTime")) {
                        Optional<ComponentConfig> componentConfig = componentConfigList.stream().filter(config ->
                                StrUtil.equalsIgnoreCase(column.getName(), config.getBindField()) ||
                                        StrUtil.equalsIgnoreCase(column.getName(), config.getBindStartTime()) ||
                                        StrUtil.equalsIgnoreCase(column.getName(), config.getBindEndTime())
                        ).findFirst();
                        componentConfig.ifPresent(config -> {
                            String format = MapUtils.getString(config.getOptions(), "format");
                            if (StrUtil.isNotEmpty(format)) {
                                fieldConfig.setPattern(LocalDateTimeUtil.convertFontFormat(format));
                            }
                        });
                    }
                    fieldConfig.setFieldComment(column.getComment());
                    fieldConfig.setPk(false);
                    fieldConfig.setFieldType(fieldType);
                    fieldConfig.setFieldName(StrUtil.toCamelCase(column.getName()));

                    fieldConfigList.add(fieldConfig);

                }

            }


            //如果有子表  每多一个子表  多加2个长度
            Map<String, Object> entityMap = new HashMap<>(8 + (tableInfos.size() - 1) * 2);
            entityMap.put(EntityConstant.PACKAGE, GlobalConstant.GENERATOR_DEFAULT_PATH + StringPool.DOT + generatorConfig.getOutputConfig().getOutputValue() + StringPool.DOT + "dto");
            entityMap.put(EntityConstant.AUTH_NAME, generatorConfig.getOutputConfig().getCreator());
            entityMap.put(EntityConstant.DATE, LocalDate.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd")));
            entityMap.put(EntityConstant.TABLE_NAME, tableInfo.getTableName());
            entityMap.put(EntityConstant.TABLE_COMMENT, generatorConfig.getOutputConfig().getComment());
            //先将表明转换为驼峰命名 然后在转换为首字母大写
            entityMap.put(EntityConstant.ENTITY_CLASS_NAME, StrUtil.upperFirst(StrUtil.toCamelCase(tableInfo.getTableName())));
            entityMap.put(EntityConstant.TABLE_FIELDS, fieldConfigList);

            entityMap.put(EntityConstant.OUTPUT_AREA, generatorConfig.getOutputConfig().getOutputValue());

            //如果是多表关联 则需要添加关联表的信息  判断数据表配置是否大于1  并且  是主表
            boolean isMainTable = generatorConfig.getTableConfigs().stream().anyMatch(x -> StrUtil.equals(x.getTableName(), tableInfo.getTableName()) && x.getIsMain());
            if (tableInfos.size() > 1 && isMainTable) {
                List<TableConfig> childTables = generatorConfig.getTableConfigs().stream()
                        .filter(x -> !x.getIsMain())
                        .collect(Collectors.toList());
                List<TableConfig> newChildTables = new ArrayList<>();
                for (TableConfig childTable : childTables) {
                    TableConfig newTableConfig = BeanUtil.toBean(childTable, TableConfig.class);

                    newTableConfig.setRelationField(StrUtil.toCamelCase(childTable.getRelationField()));
                    newTableConfig.setRelationTableField(StrUtil.toCamelCase(childTable.getRelationTableField()));
                    newTableConfig.setTableName(StrUtil.toCamelCase(childTable.getTableName()));

                    newChildTables.add(newTableConfig);
                }
//                Optional<TableConfig> parentTable = generatorConfig.getTableConfigs().stream().filter(TableConfig::getIsMain).findFirst();
                entityMap.put(EntityConstant.CHILD_TABLES, newChildTables);
//                parentTable.ifPresent(x -> entityMap.put(EntityConstant.PARENT_RELATION_FIELD_KEY, x.getRelationTableField()));
            }

            //生产目标代码
//            Configuration configuration = new Configuration(Configuration.DEFAULT_INCOMPATIBLE_IMPROVEMENTS);
//            configuration.setDefaultEncoding(StandardCharsets.UTF_8.name());
//            configuration.setDirectoryForTemplateLoading(new File(TEMPLATE_PATH));
//            Template template = configuration.getTemplate(UPDATE_DTO_TEMPLATE_NAME);
            Template template = freeMarkerConfigurer.getConfiguration().getTemplate(UPDATE_DTO_TEMPLATE_NAME);
            String entityContent = FreeMarkerTemplateUtils.processTemplateIntoString(template, entityMap);

            updateCodeMap.put("Update" + StrUtil.upperFirst(StrUtil.toCamelCase(tableInfo.getTableName())) + "Dto", entityContent);
        }

        return updateCodeMap;
    }


    /**
     * 根据配置生成 分页入参dto代码
     *
     * @param generatorConfig 入参
     * @param tableInfos      表信息
     * @param mainTable       主表配置
     * @return 代码
     */
    @SneakyThrows
    private Map<String, String> getPageDtoCode(GeneratorConfig generatorConfig, List<Table> tableInfos, TableConfig mainTable) {
        List<QueryConfig> queryConfigs = generatorConfig.getListConfig().getQueryConfigs();

        List<FieldConfig> queryFieldConfigList = new ArrayList<>();

        Optional<Table> mainTableInfoOptional = tableInfos.stream().filter(x -> StrUtil.equalsIgnoreCase(x.getTableName(), mainTable.getTableName())).findFirst();
        for (QueryConfig queryConfig : queryConfigs) {
            if (mainTableInfoOptional.isPresent()) {
                Table mainTableInfo = mainTableInfoOptional.get();
                Optional<Column> first = mainTableInfo.getColumns().stream().filter(x -> StrUtil.equalsIgnoreCase(x.getName(), queryConfig.getFieldName())).findFirst();
                first.ifPresent(x -> {
                    //如果是日期类型 默认设置查询参数为两个
                    if (queryConfig.getIsDate()) {
                        //开始时间
                        FieldConfig startConfig = new FieldConfig();
                        startConfig.setPk(false);
                        startConfig.setFieldComment(x.getComment() + "字段开始时间");
                        startConfig.setFieldName(StrUtil.toCamelCase(x.getName()) + "Start");
                        startConfig.setFieldType(JdbcToJavaUtil.getClassName(x.getTypeEnum()));
                        queryFieldConfigList.add(startConfig);

                        //结束时间
                        FieldConfig endConfig = new FieldConfig();
                        endConfig.setPk(false);
                        endConfig.setFieldComment(x.getComment() + "字段结束时间");
                        endConfig.setFieldName(StrUtil.toCamelCase(x.getName()) + "End");
                        endConfig.setFieldType(JdbcToJavaUtil.getClassName(x.getTypeEnum()));
                        queryFieldConfigList.add(endConfig);
                    } else {
                        //默认插入主键字段 放在第一个
                        FieldConfig queryFieldConfig = new FieldConfig();
                        queryFieldConfig.setPk(false);
                        queryFieldConfig.setFieldComment(x.getComment());
                        queryFieldConfig.setFieldType(JdbcToJavaUtil.getClassName(x.getTypeEnum()));
                        queryFieldConfig.setFieldName(StrUtil.toCamelCase(x.getName()));
                        queryFieldConfigList.add(queryFieldConfig);
                    }

                });

            } else {
                throw new MyException("没有主表");
            }


        }

        // 是否有左侧菜单 如果有左侧菜单  再queryconfig里面 新增一个查询字段  用于左侧菜单点击后 带参数查询
        if (generatorConfig.getListConfig().getIsLeftMenu()) {
            String fieldName = generatorConfig.getListConfig().getLeftMenuConfig().getListFieldName();

            //查询条件是否包含此字段
            boolean b = queryConfigs.stream().anyMatch(x -> StrUtil.equals(x.getFieldName(), fieldName));
            //如果未包含 则新增一个查询字段
            if (!b) {
                Optional<Table> mainTableInfoOp = tableInfos.stream().filter(x -> StrUtil.equals(x.getTableName(), mainTable.getTableName())).findFirst();
                mainTableInfoOp.ifPresent(x -> {
                    //获取当前主表的所有字段
                    Collection<Column> columns = x.getColumns();
                    //获取到当前左侧树 的查询字段 类型
                    Optional<JdbcType> leftQueryFieldType = columns.stream().filter(column -> StrUtil.equals(fieldName, column.getName())).map(Column::getTypeEnum).findFirst();

                    //默认插入左侧树查询字段
                    leftQueryFieldType.ifPresent(type -> {

                        FieldConfig queryFieldConfig = new FieldConfig();
                        queryFieldConfig.setPk(false);
                        queryFieldConfig.setFieldComment(x.getComment());
                        queryFieldConfig.setFieldType(JdbcToJavaUtil.getClassName(type));
                        queryFieldConfig.setFieldName(StrUtil.toCamelCase(fieldName));
                        queryFieldConfigList.add(queryFieldConfig);
                    });
                });

            }
        }

        Map<String, Object> queryMap = new HashMap<>(7);
        queryMap.put(EntityConstant.PACKAGE, GlobalConstant.GENERATOR_DEFAULT_PATH + StringPool.DOT + generatorConfig.getOutputConfig().getOutputValue() + StringPool.DOT + "dto");
        queryMap.put(EntityConstant.AUTH_NAME, generatorConfig.getOutputConfig().getCreator());
        queryMap.put(EntityConstant.DATE, LocalDate.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd")));
        queryMap.put(EntityConstant.TABLE_COMMENT, generatorConfig.getListConfig().getIsPage() ? "分页查询入参" : "列表查询入参");
        queryMap.put(EntityConstant.IS_PAGE, generatorConfig.getListConfig().getIsPage());
        //先将表明转换为驼峰命名 然后在转换为首字母大写
        queryMap.put(EntityConstant.ENTITY_CLASS_NAME, StrUtil.upperFirst(StrUtil.toCamelCase(mainTable.getTableName())));
        queryMap.put(EntityConstant.TABLE_FIELDS, queryFieldConfigList);

        //生产目标代码
//        Configuration configuration = new Configuration(Configuration.DEFAULT_INCOMPATIBLE_IMPROVEMENTS);
//        configuration.setDefaultEncoding(StandardCharsets.UTF_8.name());
//        configuration.setDirectoryForTemplateLoading(new File(TEMPLATE_PATH));
//        Template template = configuration.getTemplate(PAGE_DTO_TEMPLATE_NAME);
        Template template = freeMarkerConfigurer.getConfiguration().getTemplate(PAGE_DTO_TEMPLATE_NAME);

        String pageDtoConent = FreeMarkerTemplateUtils.processTemplateIntoString(template, queryMap);

        Map<String, String> map = new HashMap<>(1);
        map.put(StrUtil.upperFirst(StrUtil.toCamelCase(mainTable.getTableName())) + (generatorConfig.getListConfig().getIsPage() ? "Page" : "List") + "Dto", pageDtoConent);

        return map;
    }

    /**
     * 根据配置生成 列表页Vo代码
     *
     * @param generatorConfig 入参
     * @param tableInfos      表信息
     * @param mainTableConfig 主表配置
     * @return 代码
     */
    @SneakyThrows
    private Map<String, String> getPageVoCode(GeneratorConfig generatorConfig, List<Table> tableInfos, TableConfig mainTableConfig) {
        List<ColumnConfig> columnConfigs = generatorConfig.getListConfig().getColumnConfigs();
        //获取表的所有字段
        Optional<Table> mainTableOp = tableInfos.stream().filter(x -> StrUtil.equalsIgnoreCase(x.getTableName(), mainTableConfig.getTableName())).findFirst();

        Table mainTable;
        if (mainTableOp.isPresent()) {
            mainTable = mainTableOp.get();
        } else {
            throw new MyException("没有主表");
        }

        Collection<Column> columns = mainTable.getColumns();


        List<FieldConfig> fieldConfigList = new ArrayList<>();

        Map<String, List<ComponentConfig>> tableFieldConfigList = GeneratorUtil.buildFormComponentList(generatorConfig.getFormJson().getList());
        List<ComponentConfig> mainTableFieldConfigList = tableFieldConfigList.get(mainTable.getTableName());
        for (Column column : columns) {

            //列表配置有  就需要添加
            if (columnConfigs.stream().anyMatch(x -> StrUtil.equalsIgnoreCase(column.getName(), x.getColumnName())) || column.isPk()) {
                FieldConfig fieldConfig = new FieldConfig();
                fieldConfig.setFieldComment(column.getComment());
                fieldConfig.setPk(false);
                //如果是主键 就默认使用字符串类型 前端无法识别long类型的精度
                fieldConfig.setFieldType(column.isPk() ? "String" : JdbcToJavaUtil.getClassName(column.getTypeEnum()));
                fieldConfig.setFieldName(StrUtil.toCamelCase(column.getName()));
                // 表单配置
                if (!column.isPk()) {
                    ComponentConfig componentConfig = mainTableFieldConfigList.stream().filter(config ->
                            StrUtil.equalsIgnoreCase(column.getName(), config.getBindField()) ||
                                    StrUtil.equalsIgnoreCase(column.getName(), config.getBindStartTime()) ||
                                    StrUtil.equalsIgnoreCase(column.getName(), config.getBindEndTime())
                    ).findFirst().get();
                    String componentType = componentConfig.getType();
                    Map<String, Object> options = componentConfig.getOptions();
                    if (StrUtil.equalsIgnoreCase(componentType, ComponentTypeConstant.INFO)) {
                        Integer infoType = MapUtils.getInteger(options, "infoType");
                        String toType = null;
                        switch (infoType) {
                            case 0:
                                toType = "user";
                                break;
                            case 1:
                                toType = "organization";
                                break;
                            case 2:
                                toType = "datetime";
                                break;
                        }
                        fieldConfig.setComponentType(toType);
                    } else {
                        fieldConfig.setComponentType(componentType);
                    }
                    String datasourceType = MapUtils.getString(options, "datasourceType");
                    if (!StrUtil.isEmpty(datasourceType)) {
                        fieldConfig.setDatasourceType(datasourceType);
                        String datasourceId = null;
                        if (StrUtil.equalsIgnoreCase(datasourceType, "dic")) {
                            datasourceId = MapUtils.getString(options, "itemId");
                        } else if (StrUtil.equalsIgnoreCase(datasourceType, "api")) {
                            datasourceId = MapUtils.getString(MapUtils.getMap(options, "apiConfig"), "apiId");
                        }
                        fieldConfig.setDatasourceId(datasourceId);
                    }
                    String format = MapUtils.getString(options, "format");
                    if (StrUtil.isNotEmpty(format)) {
                        fieldConfig.setPattern(LocalDateTimeUtil.convertFontFormat(format));
                    }
                    // 多选值组件
                    if (StrUtil.equalsIgnoreCase(componentType, ComponentTypeConstant.MULTIPLE_POPUP)
                            || StrUtil.equalsIgnoreCase(componentType, ComponentTypeConstant.CHECKBOX)) {
                        fieldConfig.setMulti(true);
                    }
                    //级联
                    if (StrUtil.equalsIgnoreCase(componentType, ComponentTypeConstant.CASCADE)) {
                        fieldConfig.setSeparator(MapUtils.getString(options, ComponentTypeConstant.SEPARATOR));
                        fieldConfig.setShowFormat(MapUtils.getString(options, ComponentTypeConstant.SHOW_FORMAT));
                        fieldConfig.setDatasourceId(MapUtils.getString(MapUtils.getMap(options, "apiConfig"), "apiId"));
                    }
                    // 设置时间范围和日期范围label
                    String label = componentConfig.getLabel();
                    if (StrUtil.equalsIgnoreCase(column.getName(), componentConfig.getBindStartTime())) {
                        label += "开始";
                    } else if (StrUtil.equalsIgnoreCase(column.getName(), componentConfig.getBindEndTime())) {
                        label += "结束";
                    }
                    fieldConfig.setLabel(label);
                } else {
                    fieldConfig.setPk(true);
                }
                fieldConfigList.add(fieldConfig);
                continue;
            }
            //列表添加数据权限字段返回
            if (BooleanUtils.isTrue(generatorConfig.getOutputConfig().getIsDataAuth()) && StrUtil.equalsIgnoreCase(column.getName(), GlobalConstant.AUTH_USER_ID)) {
                FieldConfig fieldConfig = new FieldConfig();
                fieldConfig.setFieldComment(column.getComment());
                fieldConfig.setPk(false);
                //如果是主键 就默认使用字符串类型 前端无法识别long类型的精度
                fieldConfig.setFieldType(column.isPk() ? "String" : JdbcToJavaUtil.getClassName(column.getTypeEnum()));
                fieldConfig.setFieldName(StrUtil.toCamelCase(column.getName()));
                fieldConfigList.add(fieldConfig);
            }
        }


        Map<String, Object> columnMap = new HashMap<>(7);
        columnMap.put(EntityConstant.PACKAGE, GlobalConstant.GENERATOR_DEFAULT_PATH + StringPool.DOT + generatorConfig.getOutputConfig().getOutputValue() + StringPool.DOT + "vo");
        columnMap.put(EntityConstant.AUTH_NAME, generatorConfig.getOutputConfig().getCreator());
        columnMap.put(EntityConstant.DATE, LocalDate.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd")));
        columnMap.put(EntityConstant.TABLE_COMMENT, generatorConfig.getListConfig().getIsPage() ? "分页列表出参" : "列表列表入参");
        columnMap.put(EntityConstant.IS_PAGE, generatorConfig.getListConfig().getIsPage());
        //先将表明转换为驼峰命名 然后在转换为首字母大写
        columnMap.put(EntityConstant.ENTITY_CLASS_NAME, StrUtil.upperFirst(StrUtil.toCamelCase(mainTable.getTableName())));
        columnMap.put(EntityConstant.TABLE_FIELDS, fieldConfigList);
        // 导入导出
        boolean isImport = false;
        boolean isExport = false;
        List<ButtonConfig> buttonConfigs = generatorConfig.getListConfig().getButtonConfigs();
        for (ButtonConfig config : buttonConfigs) {
            if (!isImport)
                isImport = StringUtils.equals(config.getCode(), ComponentTypeConstant.BUTTON_IMPORT) && config.getIsUse();
            if (!isExport)
                isExport = StringUtils.equals(config.getCode(), ComponentTypeConstant.BUTTON_EXPORT) && config.getIsUse();
        }
        columnMap.put(EntityConstant.IS_IMPORT, isImport);
        columnMap.put(EntityConstant.IS_EXPORT, isExport);


        //生产目标代码
//        Configuration configuration = new Configuration(Configuration.DEFAULT_INCOMPATIBLE_IMPROVEMENTS);
//        configuration.setDefaultEncoding(StandardCharsets.UTF_8.name());
//        configuration.setDirectoryForTemplateLoading(new File(TEMPLATE_PATH));
//        Template template = configuration.getTemplate(PAGE_VO_TEMPLATE_NAME);

        Template template = freeMarkerConfigurer.getConfiguration().getTemplate(PAGE_VO_TEMPLATE_NAME);

        String pageVoConent = FreeMarkerTemplateUtils.processTemplateIntoString(template, columnMap);

        Map<String, String> map = new HashMap<>(1);
        map.put(StrUtil.upperFirst(StrUtil.toCamelCase(mainTable.getTableName())) + (generatorConfig.getListConfig().getIsPage() ? "Page" : "List") + "Vo", pageVoConent);

        return map;
    }

    /**
     * 根据配置生成 表单Vo代码
     *
     * @param generatorConfig 入参
     * @param tableInfos      表信息
     * @param mainTableConfig 主表配置
     * @return 代码
     */
    @SneakyThrows
    private Map<String, String> getInfoVoCode(GeneratorConfig generatorConfig, List<Table> tableInfos, TableConfig mainTableConfig) {
        //获取表的所有字段
        Optional<Table> mainTableOp = tableInfos.stream().filter(x -> StrUtil.equalsIgnoreCase(x.getTableName(), mainTableConfig.getTableName())).findFirst();

        Table mainTable;
        if (mainTableOp.isPresent()) {
            mainTable = mainTableOp.get();
        } else {
            throw new MyException("没有主表");
        }

        Map<String, String> infoVoMap = new HashMap<>(1);
        // 遍历数据表配置 根据表明搜索表信息  然后生成代码
        for (Table tableInfo : tableInfos) {
            //获取表的所有字段
            Collection<Column> columns = tableInfo.getColumns();


            List<FieldConfig> fieldConfigList = new ArrayList<>();

            for (Column column : columns) {
                FieldConfig fieldConfig = new FieldConfig();

                //只要不是审计字段都返回去
                if (!GlobalConstant.AUTO_INSERT.contains(column.getName()) &&
                        !GlobalConstant.AUTO_UPDATE.contains(column.getName()) &&
                        !StrUtil.equalsIgnoreCase(column.getName(), GlobalConstant.DELETE_MARK) &&
                        !StrUtil.equalsIgnoreCase(column.getName(), GlobalConstant.DELETE_MARK)) {

                    fieldConfig.setFieldComment(column.getComment());
                    fieldConfig.setPk(false);
                    fieldConfig.setFieldType(JdbcToJavaUtil.getClassName(column.getTypeEnum()));
                    fieldConfig.setFieldName(StrUtil.toCamelCase(column.getName()));

                    fieldConfigList.add(fieldConfig);

                }
            }


            Map<String, Object> columnMap = new HashMap<>(7 + (tableInfos.size() - 1) * 1);
            columnMap.put(EntityConstant.PACKAGE, GlobalConstant.GENERATOR_DEFAULT_PATH + StringPool.DOT + generatorConfig.getOutputConfig().getOutputValue() + StringPool.DOT + "vo");
            columnMap.put(EntityConstant.AUTH_NAME, generatorConfig.getOutputConfig().getCreator());
            columnMap.put(EntityConstant.DATE, LocalDate.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd")));
            columnMap.put(EntityConstant.TABLE_COMMENT, "表单出参");
            //先将表明转换为驼峰命名 然后在转换为首字母大写
            columnMap.put(EntityConstant.ENTITY_CLASS_NAME, StrUtil.upperFirst(StrUtil.toCamelCase(tableInfo.getTableName())));
            columnMap.put(EntityConstant.TABLE_FIELDS, fieldConfigList);

            columnMap.put(EntityConstant.OUTPUT_AREA, generatorConfig.getOutputConfig().getOutputValue());

            //如果是多表关联 则需要添加关联表的信息  判断数据表配置是否大于1  并且  是主表
            boolean isMainTable = generatorConfig.getTableConfigs().stream().anyMatch(x -> StrUtil.equals(x.getTableName(), tableInfo.getTableName()) && x.getIsMain());
            if (tableInfos.size() > 1 && isMainTable) {
                List<TableConfig> childTables = generatorConfig.getTableConfigs().stream()
                        .filter(x -> !x.getIsMain())
                        .collect(Collectors.toList());
                List<TableConfig> newChildTables = new ArrayList<>();
                for (TableConfig childTable : childTables) {
                    TableConfig newTableConfig = BeanUtil.toBean(childTable, TableConfig.class);

                    newTableConfig.setRelationField(StrUtil.toCamelCase(childTable.getRelationField()));
                    newTableConfig.setRelationTableField(StrUtil.toCamelCase(childTable.getRelationTableField()));
                    newTableConfig.setTableName(StrUtil.toCamelCase(childTable.getTableName()));

                    newChildTables.add(newTableConfig);
                }
//                Optional<TableConfig> parentTable = generatorConfig.getTableConfigs().stream().filter(TableConfig::getIsMain).findFirst();
                columnMap.put(EntityConstant.CHILD_TABLES, newChildTables);
//                parentTable.ifPresent(x -> columnMap.put(EntityConstant.PARENT_RELATION_FIELD_KEY, x.getRelationTableField()));
            }
            //生产目标代码
//            Configuration configuration = new Configuration(Configuration.DEFAULT_INCOMPATIBLE_IMPROVEMENTS);
//            configuration.setDefaultEncoding(StandardCharsets.UTF_8.name());
//            configuration.setDirectoryForTemplateLoading(new File(TEMPLATE_PATH));
//            Template template = configuration.getTemplate(INFO_VO_TEMPLATE_NAME);

            Template template = freeMarkerConfigurer.getConfiguration().getTemplate(INFO_VO_TEMPLATE_NAME);

            String infoVoConent = FreeMarkerTemplateUtils.processTemplateIntoString(template, columnMap);


            infoVoMap.put(StrUtil.upperFirst(StrUtil.toCamelCase(tableInfo.getTableName())) + "Vo", infoVoConent);

        }

        return infoVoMap;
    }

    /**
     * 根据配置生成mapper文件
     *
     * @param generatorConfig 入参
     * @return 代码
     */
    @SneakyThrows
    private Map<String, String> getMapperCode(GeneratorConfig generatorConfig, List<Table> tableInfos) {


        Map<String, String> map = new HashMap<>(1 + tableInfos.size() - 1);

        for (Table tableInfo : tableInfos) {

            Map<String, Object> mapperMap = new HashMap<>(7);
            mapperMap.put(EntityConstant.PACKAGE, GlobalConstant.GENERATOR_DEFAULT_PATH + StringPool.DOT + generatorConfig.getOutputConfig().getOutputValue() + StringPool.DOT + "mapper");
            mapperMap.put(EntityConstant.AUTH_NAME, generatorConfig.getOutputConfig().getCreator());
            mapperMap.put(EntityConstant.DATE, LocalDate.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd")));
            mapperMap.put(EntityConstant.TABLE_COMMENT, "mapper");
            //先将表明转换为驼峰命名 然后在转换为首字母大写
            mapperMap.put(EntityConstant.ENTITY_CLASS_NAME, StrUtil.upperFirst(StrUtil.toCamelCase(tableInfo.getTableName())));
            mapperMap.put(EntityConstant.FUNCTION_CLASS_NAME, StrUtil.upperFirst(StrUtil.toCamelCase(generatorConfig.getOutputConfig().getClassName())));
            mapperMap.put(EntityConstant.OUTPUT_AREA, generatorConfig.getOutputConfig().getOutputValue());

            mapperMap.put(EntityConstant.IS_MULTI, tableInfos.size() > 1);

            //生产目标代码
//            Configuration configuration = new Configuration(Configuration.DEFAULT_INCOMPATIBLE_IMPROVEMENTS);
//            configuration.setDefaultEncoding(StandardCharsets.UTF_8.name());
//            configuration.setDirectoryForTemplateLoading(new File(TEMPLATE_PATH));
//            Template template = configuration.getTemplate(MAPPPER_TEMPLATE_NAME);

            Template template = freeMarkerConfigurer.getConfiguration().getTemplate(MAPPPER_TEMPLATE_NAME);

            String mapperContent = FreeMarkerTemplateUtils.processTemplateIntoString(template, mapperMap);

            map.put(StrUtil.upperFirst(StrUtil.toCamelCase(tableInfo.getTableName())) + "Mapper", mapperContent);

        }


        return map;
    }

    /**
     * 根据配置生成service文件
     *
     * @param generatorConfig 入参
     * @param mainTableConfig 主表配置
     * @return 代码
     */
    @SneakyThrows
    private Map<String, String> getServiceCode(GeneratorConfig generatorConfig, TableConfig mainTableConfig, Boolean isMulti) {
        Map<String, Object> serviceMap = new HashMap<>(7);
        serviceMap.put(EntityConstant.PACKAGE, GlobalConstant.GENERATOR_DEFAULT_PATH + StringPool.DOT + generatorConfig.getOutputConfig().getOutputValue() + StringPool.DOT + "service");
        serviceMap.put(EntityConstant.AUTH_NAME, generatorConfig.getOutputConfig().getCreator());
        serviceMap.put(EntityConstant.DATE, LocalDate.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd")));
        serviceMap.put(EntityConstant.TABLE_COMMENT, "service");
        //先将表明转换为驼峰命名 然后在转换为首字母大写
        serviceMap.put(EntityConstant.ENTITY_CLASS_NAME, StrUtil.upperFirst(StrUtil.toCamelCase(mainTableConfig.getTableName())));
        serviceMap.put(EntityConstant.OUTPUT_AREA, generatorConfig.getOutputConfig().getOutputValue());
        serviceMap.put(EntityConstant.FUNCTION_CLASS_NAME, StrUtil.upperFirst(StrUtil.toCamelCase(generatorConfig.getOutputConfig().getClassName())));
        serviceMap.put(EntityConstant.IS_MULTI, isMulti);

        //生产目标代码
//        Configuration configuration = new Configuration(Configuration.DEFAULT_INCOMPATIBLE_IMPROVEMENTS);
//        configuration.setDefaultEncoding(StandardCharsets.UTF_8.name());
//        configuration.setDirectoryForTemplateLoading(new File(TEMPLATE_PATH));
//        Template template = configuration.getTemplate(SERVICE_TEMPLATE_NAME);

        Template template = freeMarkerConfigurer.getConfiguration().getTemplate(SERVICE_TEMPLATE_NAME);

        String serviceContent = FreeMarkerTemplateUtils.processTemplateIntoString(template, serviceMap);

        Map<String, String> map = new HashMap<>(1);
        map.put("I" + StrUtil.upperFirst(StrUtil.toCamelCase(generatorConfig.getOutputConfig().getClassName())) + "Service", serviceContent);
        return map;
    }


    /**
     * 根据配置生成service impl代码
     *
     * @param generatorConfig 入参
     * @param mainTableConfig 主表配置
     * @return 代码
     */
    @SneakyThrows
    private Map<String, String> getServiceImplCode(GeneratorConfig generatorConfig, TableConfig mainTableConfig, Boolean isMulti) {
        Optional<TableConfig> first = generatorConfig.getTableConfigs().stream().filter(TableConfig::getIsMain).findFirst();
        if (!first.isPresent()) {
            throw new MyException("未设置主表！");
        }

        Map<String, Object> serviceMap = new HashMap<>(8);
        serviceMap.put(EntityConstant.PACKAGE, GlobalConstant.GENERATOR_DEFAULT_PATH + StringPool.DOT + generatorConfig.getOutputConfig().getOutputValue() + StringPool.DOT + "service.impl");
        serviceMap.put(EntityConstant.AUTH_NAME, generatorConfig.getOutputConfig().getCreator());
        serviceMap.put(EntityConstant.DATE, LocalDate.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd")));
        serviceMap.put(EntityConstant.TABLE_COMMENT, "service");
        serviceMap.put(EntityConstant.PARENT_TABLE_KEY, first.get().getPkField());
        //先将表明转换为驼峰命名 然后在转换为首字母大写
        serviceMap.put(EntityConstant.ENTITY_CLASS_NAME, StrUtil.upperFirst(StrUtil.toCamelCase(mainTableConfig.getTableName())));
        serviceMap.put(EntityConstant.OUTPUT_AREA, generatorConfig.getOutputConfig().getOutputValue());
        serviceMap.put(EntityConstant.FUNCTION_CLASS_NAME, StrUtil.upperFirst(StrUtil.toCamelCase(generatorConfig.getOutputConfig().getClassName())));

        serviceMap.put(EntityConstant.IS_MULTI, isMulti);
        serviceMap.put(EntityConstant.DATASOURCE_ID, generatorConfig.getDatabaseId());

        //传入所有子表
        List<TableConfig> childTables = generatorConfig.getTableConfigs().stream().filter(x -> !x.getIsMain()).collect(Collectors.toList());

        List<TableConfig> newChildTables = new ArrayList<>();
        for (TableConfig childTable : childTables) {
            TableConfig newTableConfig = BeanUtil.toBean(childTable, TableConfig.class);

            newTableConfig.setRelationField(StrUtil.toCamelCase(childTable.getRelationField()));
            newTableConfig.setRelationTableField(StrUtil.toCamelCase(childTable.getRelationTableField()));
            newTableConfig.setTableName(StrUtil.toCamelCase(childTable.getTableName()));
            newTableConfig.setPkField(StrUtil.toCamelCase(childTable.getPkField()));
            newTableConfig.setPkType(childTable.getPkType());

            newChildTables.add(newTableConfig);
        }

        serviceMap.put(EntityConstant.CHILD_TABLES, newChildTables);

        //生产目标代码
//        Configuration configuration = new Configuration(Configuration.DEFAULT_INCOMPATIBLE_IMPROVEMENTS);
//        configuration.setDefaultEncoding(StandardCharsets.UTF_8.name());
//        configuration.setDirectoryForTemplateLoading(new File(TEMPLATE_PATH));
//        Template template = configuration.getTemplate(SERVICE_IMPL_TEMPLATE_NAME);

        Template template = freeMarkerConfigurer.getConfiguration().getTemplate(SERVICE_IMPL_TEMPLATE_NAME);

        String serviceImplContent = FreeMarkerTemplateUtils.processTemplateIntoString(template, serviceMap);

        Map<String, String> map = new HashMap<>(1);
        map.put(StrUtil.upperFirst(StrUtil.toCamelCase(generatorConfig.getOutputConfig().getClassName())) + "ServiceImpl", serviceImplContent);
        return map;
    }


    /**
     * 根据配置生成控制器代码
     *
     * @param generatorConfig 入参
     * @param mainTableConfig 主表配置
     * @return 代码
     */
    @SneakyThrows
    private Map<String, String> getControllerCode(GeneratorConfig generatorConfig, List<Table> tableInfos, TableConfig mainTableConfig) {

        List<QueryConfig> queryConfigs = generatorConfig.getListConfig().getQueryConfigs();

        List<FieldConfig> queryFieldConfigList = new ArrayList<>();
        for (QueryConfig queryConfig : queryConfigs) {

            //如果是日期类型 默认设置查询参数为两个
            if (queryConfig.getIsDate()) {

                FieldConfig dateConfig = new FieldConfig();
                dateConfig.setPk(false);
                dateConfig.setFieldComment(queryConfig.getFieldName() + "字段");
                dateConfig.setFieldName(StrUtil.toCamelCase(queryConfig.getFieldName()));
                dateConfig.setFieldType(LocalDateTime.class.getName().replace("java.time.", ""));
                queryFieldConfigList.add(dateConfig);
            } else {

                Optional<Table> mainTableInfoOptional = tableInfos.stream().filter(x -> StrUtil.equalsIgnoreCase(x.getTableName(), mainTableConfig.getTableName())).findFirst();
                if (mainTableInfoOptional.isPresent()) {
                    Table mainTableInfo = mainTableInfoOptional.get();
                    Optional<Column> first = mainTableInfo.getColumns().stream().filter(x -> StrUtil.equalsIgnoreCase(x.getName(), queryConfig.getFieldName())).findFirst();

                    if (first.isPresent()) {
                        Column x = first.get();
                        //默认插入主键字段 放在第一个
                        FieldConfig fieldConfig = new FieldConfig();
                        fieldConfig.setPk(x.isPk());
                        fieldConfig.setFieldComment(x.getComment());
                        fieldConfig.setFieldType(JdbcToJavaUtil.getClassName(x.getTypeEnum()));
                        String fieldName = StrUtil.toCamelCase(x.getName());
                        fieldConfig.setFieldName(fieldName);
                        queryFieldConfigList.add(fieldConfig);
                    }

                } else {
                    throw new MyException("没有主表");
                }


            }
        }

        // 是否有左侧菜单 如果有左侧菜单  再queryconfig里面 新增一个查询字段  用于左侧菜单点击后 带参数查询
        if (generatorConfig.getListConfig().getIsLeftMenu()) {
            String fieldName = generatorConfig.getListConfig().getLeftMenuConfig().getListFieldName();

            //查询条件是否包含此字段
            boolean b = queryConfigs.stream().anyMatch(x -> StrUtil.equals(x.getFieldName(), fieldName));
            //如果未包含 则新增一个查询字段
            if (!b) {
                Optional<Table> mainTableInfoOp = tableInfos.stream().filter(x -> StrUtil.equals(x.getTableName(), mainTableConfig.getTableName())).findFirst();
                mainTableInfoOp.ifPresent(x -> {
                    //获取当前主表的所有字段
                    Collection<Column> columns = x.getColumns();
                    //获取到当前左侧树 的查询字段 类型
                    Optional<JdbcType> leftQueryFieldType = columns.stream().filter(column -> StrUtil.equals(fieldName, column.getName())).map(Column::getTypeEnum).findFirst();

                    //默认插入左侧树查询字段
                    leftQueryFieldType.ifPresent(type -> {

                        FieldConfig queryFieldConfig = new FieldConfig();
                        queryFieldConfig.setPk(false);
                        queryFieldConfig.setFieldComment(x.getComment());
                        queryFieldConfig.setFieldType(JdbcToJavaUtil.getClassName(type));
                        queryFieldConfig.setFieldName(StrUtil.toCamelCase(fieldName));
                        queryFieldConfigList.add(queryFieldConfig);
                    });
                });

            }
        }
        // 找出自动编码code
        StringBuilder autoCode = new StringBuilder();
        Map<String, List<ComponentConfig>> componentListMap = GeneratorUtil.buildFormComponentList(generatorConfig.getFormJson().getList());
        for (List<ComponentConfig> componentList : componentListMap.values()) {
            for (ComponentConfig componentConfig : componentList) {
                if (StringUtils.equalsIgnoreCase(componentConfig.getType(), ComponentTypeConstant.AUTO_CODE)) {
                    String codeRule = MapUtils.getString(componentConfig.getOptions(), ComponentTypeConstant.AUTO_CODE_RULE);
                    if (StringUtils.isNotEmpty(codeRule)) {
                        autoCode.append(codeRule);
                        autoCode.append(StringPool.COMMA);
                    }
                }
            }
        }
        if (autoCode.length() > 1) autoCode.deleteCharAt(autoCode.length() - 1);

        Map<String, Object> controllerMap = new HashMap<>(12);
        controllerMap.put(EntityConstant.PACKAGE, GlobalConstant.GENERATOR_DEFAULT_PATH + StringPool.DOT + generatorConfig.getOutputConfig().getOutputValue() + StringPool.DOT + "controller");
        controllerMap.put(EntityConstant.AUTH_NAME, generatorConfig.getOutputConfig().getCreator());
        controllerMap.put(EntityConstant.DATE, LocalDate.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd")));
        controllerMap.put(EntityConstant.TABLE_COMMENT, generatorConfig.getOutputConfig().getComment());
        controllerMap.put(EntityConstant.IS_PAGE, generatorConfig.getListConfig().getIsPage());
        //先将表名转换为驼峰命名 然后在转换为首字母大写
        controllerMap.put(EntityConstant.ENTITY_CLASS_NAME, StrUtil.upperFirst(StrUtil.toCamelCase(mainTableConfig.getTableName())));
        controllerMap.put(EntityConstant.OUTPUT_AREA, generatorConfig.getOutputConfig().getOutputValue());
        controllerMap.put(EntityConstant.TABLE_FIELDS, queryFieldConfigList);
        controllerMap.put(EntityConstant.PK_FIELD, StrUtil.toCamelCase(mainTableConfig.getPkField()));
        controllerMap.put(EntityConstant.IS_MULTI, tableInfos.size() > 1);
        controllerMap.put(EntityConstant.CODE_RULES, autoCode.toString());

        controllerMap.put(EntityConstant.FUNCTION_CLASS_NAME, StrUtil.upperFirst(StrUtil.toCamelCase(generatorConfig.getOutputConfig().getClassName())));

        String orderType = null;
        String orderField = null;
        if (generatorConfig.getListConfig().getDefaultOrder()) {
            orderType = "desc";
            orderField = StrUtil.toCamelCase(mainTableConfig.getPkField());
        } else {
            orderType = generatorConfig.getListConfig().getOrderType();
            orderField = StrUtil.toCamelCase(generatorConfig.getListConfig().getOrderBy());
        }
        controllerMap.put(EntityConstant.ORDER_TYPE, StrUtil.equalsIgnoreCase(orderType, "desc"));
        controllerMap.put(EntityConstant.ORDER_FIELD, StrUtil.upperFirst(orderField));
        controllerMap.put(EntityConstant.IS_DATA_AUTH, BooleanUtils.isTrue(generatorConfig.getOutputConfig().getIsDataAuth()));
        // 导入导出
        boolean isImport = false;
        boolean isExport = false;
        List<ButtonConfig> buttonConfigs = generatorConfig.getListConfig().getButtonConfigs();
        for (ButtonConfig config : buttonConfigs) {
            if (!isImport)
                isImport = StringUtils.equals(config.getCode(), ComponentTypeConstant.BUTTON_IMPORT) && config.getIsUse();
            if (!isExport)
                isExport = StringUtils.equals(config.getCode(), ComponentTypeConstant.BUTTON_EXPORT) && config.getIsUse();
        }
        controllerMap.put(EntityConstant.IS_IMPORT, isImport);
        controllerMap.put(EntityConstant.IS_EXPORT, isExport);

        //生产目标代码
//        Configuration configuration = new Configuration(Configuration.DEFAULT_INCOMPATIBLE_IMPROVEMENTS);
//        configuration.setDefaultEncoding(StandardCharsets.UTF_8.name());
//        configuration.setDirectoryForTemplateLoading(new File(TEMPLATE_PATH));
//        Template template = configuration.getTemplate(CONTROLLER_TEMPLATE_NAME);

        Template template = freeMarkerConfigurer.getConfiguration().getTemplate(CONTROLLER_TEMPLATE_NAME);

        String controllerContent = FreeMarkerTemplateUtils.processTemplateIntoString(template, controllerMap);

        Map<String, String> map = new HashMap<>(1);
        map.put(StrUtil.upperFirst(StrUtil.toCamelCase(generatorConfig.getOutputConfig().getClassName())) + "Controller", controllerContent);
        return map;
    }


    /**
     * 获取生成目录 默认生成到src/main/java/com/xjrsoft/module
     *
     * @return
     */
    private String getOutputDir() {
        return System.getProperty("user.dir") + StringPool.SLASH + "src" + StringPool.SLASH + "main" + StringPool.SLASH + "java" + StringPool.SLASH + "com" + StringPool.SLASH + "xjrsoft" + StringPool.SLASH + "module";
    }

    /**
     * 获取实体类生成目录
     *
     * @param outputArea
     * @return
     */
    private String getEntityOutputDir(String outputArea) {
        return getOutputDir() + StringPool.SLASH + outputArea.toLowerCase() + StringPool.SLASH + "entity";
    }

    /**
     * 获取dto的生成目录
     *
     * @param outputArea
     * @return
     */
    private String getDtoOutputDir(String outputArea) {
        return getOutputDir() + StringPool.SLASH + outputArea.toLowerCase() + StringPool.SLASH + "dto";
    }

    /**
     * 获取vo的输出目录
     *
     * @param outputArea
     * @return
     */
    private String getVoOutputDir(String outputArea) {
        return getOutputDir() + StringPool.SLASH + outputArea.toLowerCase() + StringPool.SLASH + "vo";
    }

    /**
     * 获取mapper的输出目录
     *
     * @param outputArea
     * @return
     */
    private String getMapperOutputDir(String outputArea) {
        return getOutputDir() + StringPool.SLASH + outputArea.toLowerCase() + StringPool.SLASH + "mapper";
    }

    /**
     * 获取service的生成目录
     *
     * @param outputArea
     * @return
     */
    private String getServiceOutputDir(String outputArea) {
        return getOutputDir() + StringPool.SLASH + outputArea.toLowerCase() + StringPool.SLASH + "service";
    }

    /**
     * 获取serviceImpl的生成目录
     *
     * @param outputArea
     * @return
     */
    private String getServiceImplOutputDir(String outputArea) {
        return getOutputDir() + StringPool.SLASH + outputArea.toLowerCase() + StringPool.SLASH + "service" + StringPool.SLASH + "impl";
    }

    /**
     * 获取控制器生成目录
     *
     * @param outputArea
     * @return
     */
    private String getControllerOutputDir(String outputArea) {
        return getOutputDir() + StringPool.SLASH + outputArea.toLowerCase() + StringPool.SLASH + "controller";
    }

    /**
     * 获取列表页面生成路径
     *
     * @param outputArea
     * @return
     */
    private String getListOutputDir(String outputArea, String className) {
        return generatePathConfig.getWebPath() + StringPool.SLASH + "src" + StringPool.SLASH + "views" + StringPool.SLASH + outputArea.toLowerCase() + StringPool.SLASH + className.toLowerCase();
    }

    /**
     * 获取APP列表页面生成路径
     *
     * @param outputArea
     * @return
     */
    private String getAppListOutputDir(String outputArea, String className) {
        return generatePathConfig.getAppPath() + StringPool.SLASH + "pages" + StringPool.SLASH + outputArea.toLowerCase() + StringPool.SLASH + className.toLowerCase();
    }

    /**
     * 获取表单页面生成路径（表单前端是一个组件 所以 多了一层components）
     *
     * @param outputArea
     * @return
     */
    private String getFormOutputDir(String outputArea, String className) {
        return generatePathConfig.getWebPath() + StringPool.SLASH + "src" + StringPool.SLASH + "views" + StringPool.SLASH + outputArea.toLowerCase() + StringPool.SLASH + className.toLowerCase() + StringPool.SLASH + "components";
    }

    /**
     * 获取APP表单页生成路劲
     *
     * @param outputArea
     * @return
     */
    private String getAppFormOutputDir(String outputArea, String className) {
        return generatePathConfig.getAppPath() + StringPool.SLASH + "pages" + StringPool.SLASH + outputArea.toLowerCase() + StringPool.SLASH + className.toLowerCase();
    }

    /**
     * 获取前端模型生成路径
     *
     * @param outputArea
     * @return
     */
    private String getModelOutputDir(String outputArea, String className) {
        return generatePathConfig.getWebPath() + StringPool.SLASH + "src" + StringPool.SLASH + "api" + StringPool.SLASH + outputArea.toLowerCase() + StringPool.SLASH + className.toLowerCase() + StringPool.SLASH + "model";
    }

    /**
     * 获取前端请求生成路径
     *
     * @param outputArea
     * @return
     */
    private String getApiOutputDir(String outputArea, String className) {
        return generatePathConfig.getWebPath() + StringPool.SLASH + "src" + StringPool.SLASH + "api" + StringPool.SLASH + outputArea.toLowerCase() + StringPool.SLASH + className.toLowerCase();
    }

    /**
     * 获取APP请求生成路径
     *
     * @param outputArea
     * @return
     */
    private String getAppApiOutputDir(String outputArea, String className) {
        return generatePathConfig.getAppPath() + StringPool.SLASH + "common" + StringPool.SLASH + "api" + StringPool.SLASH + outputArea.toLowerCase() + StringPool.SLASH + className.toLowerCase();
    }


    /**
     * 获取APP配置请求生成路径
     *
     * @param outputArea
     * @return
     */
    private String getAppConfigOutputDir(String outputArea, String className) {
        return generatePathConfig.getAppPath() + StringPool.SLASH + "pages" + StringPool.SLASH + outputArea.toLowerCase() + StringPool.SLASH + className.toLowerCase() + StringPool.SLASH + "config";
    }


    /**
     * 默认插入菜单
     *
     * @param generatorConfig
     */
    private Menu insertMenuConfig(GeneratorConfig generatorConfig) {
        MenuConfig menuConfig = generatorConfig.getMenuConfig();
        Menu menu = BeanUtil.toBean(menuConfig, Menu.class);
        Menu oldMenu = menuMapper.selectOne(Wrappers.lambdaQuery(Menu.class).select(Menu::getId).eq(Menu::getName, menuConfig.getCode()));
        if (oldMenu != null) {
            menu.setId(oldMenu.getId());
            menu.setFormId(oldMenu.getFormId());
        } else {
            menu.setFormId(IdUtil.getSnowflakeNextId());
        }
        menu.setTitle(menuConfig.getName());
        menu.setComponent(StringPool.SLASH + generatorConfig.getOutputConfig().getOutputValue().toLowerCase() + StringPool.SLASH + generatorConfig.getOutputConfig().getClassName().toLowerCase() + StringPool.SLASH + "index");
        menu.setMenuType(MenuType.FUNCTION.getCode());
        menu.setDisplay(1);
        menu.setAllowDelete(1);
        menu.setAllowModify(1);
        menu.setOutLink(0);
        menu.setKeepAlive(0);
        menu.setSortCode(menuConfig.getSortCode());
        menu.setPath(PinyinUtil.getPinyin(menuConfig.getName(), StringPool.EMPTY).toLowerCase());
        //保存菜单
        if (oldMenu != null) {
            menuMapper.updateById(menu);
        } else {
            menuMapper.insert(menu);
        }

        Long menuId = menu.getId();
        List<ButtonConfig> buttonConfigs = generatorConfig.getListConfig().getButtonConfigs();
        List<MenuButton> oldBtnList = menuButtonService.list(Wrappers.lambdaQuery(MenuButton.class)
                .select(MenuButton::getId, MenuButton::getCode).eq(MenuButton::getMenuId, menuId));
        Map<String, Long> btnCodeIdMap = oldBtnList.stream().collect(Collectors.toMap(m ->
                StringUtils.defaultIfBlank(StringUtils.substringAfterLast(m.getCode(), StringPool.COLON), m.getCode()), MenuButton::getId));
        List<MenuButton> btnList = new ArrayList<>();
        buttonConfigs.forEach(buttonConfig -> {
            if (BooleanUtils.isTrue(buttonConfig.getIsUse())) {
                String code = generatorConfig.getOutputConfig().getClassName().toLowerCase() + StringPool.COLON + buttonConfig.getCode();
                MenuButton menuButton = new MenuButton();
                menuButton.setId(btnCodeIdMap.get(StringUtils.defaultIfBlank(StringUtils.substringAfterLast(code, StringPool.COLON), code)));
                menuButton.setMenuId(menuId);
                menuButton.setName(buttonConfig.getName());
                menuButton.setCode(code);
                menuButton.setIcon(buttonConfig.getIcon());
                btnList.add(menuButton);
            }
        });

        List<ColumnConfig> columnConfigs = generatorConfig.getListConfig().getColumnConfigs();
        List<MenuColumn> oldColList = menuColumnService.list(Wrappers.lambdaQuery(MenuColumn.class)
                .select(MenuColumn::getId, MenuColumn::getCode).eq(MenuColumn::getMenuId, menuId));
        Map<String, Long> colCodeIdMap = oldColList.stream().collect(Collectors.toMap(MenuColumn::getCode, MenuColumn::getId));
        List<MenuColumn> colList = new ArrayList<>();
        for (ColumnConfig columnConfig : columnConfigs) {
            MenuColumn menuColumn = new MenuColumn();
            menuColumn.setId(colCodeIdMap.get(StrUtil.toCamelCase(columnConfig.getColumnName())));
            menuColumn.setMenuId(menuId);
            menuColumn.setName(columnConfig.getLabel());
            menuColumn.setCode(StrUtil.toCamelCase(columnConfig.getColumnName()));
            colList.add(menuColumn);
        }
        List<ComponentConfig> componentConfigList = GeneratorUtil.getFormComponentListWithoutLayout(generatorConfig.getFormJson().getList());
        List<MenuForm> oldFormList = menuFormService.list(Wrappers.lambdaQuery(MenuForm.class)
                .select(MenuForm::getId, MenuForm::getCode).eq(MenuForm::getMenuId, menuId));
        Map<String, Long> formCodeIdMap = oldFormList.stream().collect(Collectors.toMap(m -> m.getParentId() + StringPool.AMPERSAND + m.getCode(), MenuForm::getId));
        List<MenuForm> formList = new ArrayList<>();
        for (ComponentConfig componentConfig : componentConfigList) {
            String type = componentConfig.getType();
            if (StrUtil.equalsIgnoreCase(type, ComponentTypeConstant.SUB_FORM) || StrUtil.equalsIgnoreCase(type, ComponentTypeConstant.ONE_FOR_ONE_FORM)) {
                Long oldId = formCodeIdMap.get(StringPool.ZERO + StringPool.AMPERSAND + StrUtil.toCamelCase(componentConfig.getBindTable()));
                long id = oldId == null ? IdUtil.getSnowflakeNextId() : oldId;
                IdWorker.getId();
                MenuForm menuForm = new MenuForm();
                menuForm.setId(id);
                menuForm.setMenuId(menuId);
                menuForm.setCode(StrUtil.toCamelCase(componentConfig.getBindTable()));
                menuForm.setName(componentConfig.getLabel());
                formList.add(menuForm);
                for (ComponentConfig subConfig : componentConfig.getChildren()) {
                    MenuForm form = buildMenuForm(id, menuId, subConfig);
                    form.setId(formCodeIdMap.get(id + StringPool.AMPERSAND + form.getCode()));
                    formList.add(form);
                }
            } else {
                MenuForm form = buildMenuForm(0L, menuId, componentConfig);
                form.setId(formCodeIdMap.get(StringPool.ZERO + StringPool.AMPERSAND + form.getCode()));
                formList.add(form);
            }
        }
        menuButtonService.remove(Wrappers.lambdaQuery(MenuButton.class).eq(MenuButton::getMenuId, menuId));
        menuButtonService.saveBatch(btnList);
        menuColumnService.remove(Wrappers.lambdaQuery(MenuColumn.class).eq(MenuColumn::getMenuId, menuId));
        menuColumnService.saveBatch(colList);
        menuFormService.remove(Wrappers.lambdaQuery(MenuForm.class).eq(MenuForm::getMenuId, menuId));
        menuFormService.saveBatch(formList);
        return menu;
    }

    private MenuForm buildMenuForm(Long parentId, Long menuId, ComponentConfig componentConfig) {
        String field = null;
        String type = componentConfig.getType();
        if (StrUtil.equalsIgnoreCase(type, ComponentTypeConstant.DATE_RANGE)
                || StrUtil.equalsIgnoreCase(type, ComponentTypeConstant.TIME_RANGE)) {
            field = componentConfig.getBindStartTime() + StringPool.COMMA + componentConfig.getBindEndTime();
        } else {
            field = StrUtil.isEmpty(componentConfig.getBindField()) ? componentConfig.getKey() : StrUtil.toCamelCase(componentConfig.getBindField());
        }
        Boolean isRequired = MapUtils.getBoolean(componentConfig.getOptions(), "required", Boolean.FALSE);
        MenuForm subMenuForm = new MenuForm();
        subMenuForm.setParentId(parentId);
        subMenuForm.setMenuId(menuId);
        subMenuForm.setCode(field);
        subMenuForm.setName(componentConfig.getLabel());
        subMenuForm.setIsRequired(isRequired ? YesOrNoEnum.YES.getCode() : YesOrNoEnum.NO.getCode());
        return subMenuForm;
    }

    /**
     * 获取所有表
     *
     * @return
     */
    private MyTableInfo getTableInfo(String id, String tableName) {
        Connection connection = null;
        ResultSet resultSet = null;
        try {
            DataSource dataSource = DatasourceUtil.getDataSource(id);
            connection = dataSource.getConnection();
            DatabaseMetaData metaData = connection.getMetaData();
            resultSet = metaData.getTables(null, null, tableName, new String[]{"TABLE"});

            MyTableInfo myTableInfo = null;
            //可能会返回重复的表名  mysql 5.5 以后的驱动url都必须设置参数  &nullCatalogMeansCurrent=true  可以解决返回重复表问题
            while (resultSet.next()) {
                myTableInfo = new MyTableInfo();

                myTableInfo.setTableName(resultSet.getString("TABLE_NAME"));
                myTableInfo.setTableComment(resultSet.getString("REMARKS"));

                ResultSet columns = metaData.getColumns(null, null, tableName, null);
                ResultSet primaryKeys = metaData.getPrimaryKeys(null, null, tableName);

                //获取所有主键
                List<String> primaryKeysList = new ArrayList<>();
                while (primaryKeys.next()) {
                    primaryKeysList.add(primaryKeys.getString("PK_NAME"));
                }


                List<MyColumnInfo> columnInfoList = new ArrayList<>();
                while (columns.next()) {
                    String columnName = columns.getString("COLUMN_NAME");
                    //可能会有重复的返回值，所以需要去重
                    if (columnInfoList.stream().noneMatch(x -> StrUtil.equals(x.getColumn(), columnName))) {
                        MyColumnInfo myColumnInfo = new MyColumnInfo();
                        myColumnInfo.setColumn(columns.getString("COLUMN_NAME"));
                        myColumnInfo.setProperty(StrUtil.toCamelCase(columns.getString("COLUMN_NAME")));
                        myColumnInfo.setDataType(columns.getString("DATA_TYPE"));
                        myColumnInfo.setColumnComment(columns.getString("REMARKS"));
                        myColumnInfo.setNullable(columns.getBoolean("NULLABLE"));
                        myColumnInfo.setAutoIncrement(columns.getBoolean("IS_AUTOINCREMENT"));
                        myColumnInfo.setDataLength(columns.getInt("COLUMN_SIZE"));
                        myColumnInfo.setPrimaryKey(primaryKeysList.stream().anyMatch(x -> StrUtil.equals(x, columnName)));

                        columnInfoList.add(myColumnInfo);
                    }


                }

            }
            return myTableInfo;

        } catch (SQLException e) {
            throw new MyException("获取数据源所有表失败");
        } finally {
            DatasourceUtil.close(connection, resultSet);
        }
    }

    private boolean saveCodeSchema(Long id, OutputConfig outputConfig, String content) {
        CodeSchema codeSchema = new CodeSchema();
        codeSchema.setId(id);
        codeSchema.setCategory(Long.parseLong(outputConfig.getOutputArea()));
        codeSchema.setContent(content);
        codeSchema.setDescription(outputConfig.getComment());
        codeSchema.setRemark(outputConfig.getRemarks());
        codeSchema.setName(outputConfig.getClassName());
        codeSchema.setStatus(1);
        codeSchema.setType(outputConfig.getType());
        return codeSchemaService.saveOrUpdate(codeSchema);
    }

    private void writeFile(String dirPath, String fileName, String content) throws FileNotFoundException {
        File dir = new File(dirPath);
        if (!dir.exists()) {
            boolean mkdirs = dir.mkdirs();
        }
        String filePath = dirPath + StringPool.SLASH + fileName;
        File javaFile = new File(filePath);
        if (javaFile.exists()) {
            javaFile.renameTo(new File(dirPath + StringPool.SLASH + fileName + StringPool.DOT + System.currentTimeMillis() + ".bak"));
        }
        FileOutputStream voFile = new FileOutputStream(filePath);
        IoUtil.write(voFile, true, content.getBytes(StandardCharsets.UTF_8));
    }
}
