package com.xjrsoft.module.generator.utils;

import cn.hutool.core.util.StrUtil;
import com.xjrsoft.common.model.generator.ComponentConfig;
import com.xjrsoft.common.model.generator.LayoutOptionModel;
import com.xjrsoft.module.generator.constant.ComponentTypeConstant;
import org.apache.commons.lang3.StringUtils;
import org.apache.tomcat.util.collections.CaseInsensitiveKeyMap;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public final class GeneratorUtil {
    private GeneratorUtil(){}

    public static Map<String, List<ComponentConfig>> buildFormComponentList(List<ComponentConfig> sourceList) {
        Map<String, List<ComponentConfig>> resultMap = new CaseInsensitiveKeyMap<>();
        buildFormComponentList(resultMap, sourceList);
        return resultMap;
    }

    public static void buildFormComponentList(Map<String, List<ComponentConfig>> targetMap, List<ComponentConfig> sourceList) {
        for (ComponentConfig config : sourceList) {
            String type = config.getType();
            if (StringUtils.equalsIgnoreCase(type, ComponentTypeConstant.SUB_FORM) || StrUtil.equalsIgnoreCase(type, ComponentTypeConstant.ONE_FOR_ONE_FORM)) {
                List<ComponentConfig> children = config.getChildren();
                buildFormComponentList(targetMap, children);
            } else if (StringUtils.equalsIgnoreCase(type, ComponentTypeConstant.TAB)
                    || StringUtils.equalsIgnoreCase(type, ComponentTypeConstant.CARD)
                    || StringUtils.equalsIgnoreCase(type, ComponentTypeConstant.GRID)) {
                List<LayoutOptionModel> tabList = config.getLayout();
                for (LayoutOptionModel tab : tabList) {
                    List<ComponentConfig> tabComponentList = tab.getList();
                    buildFormComponentList(targetMap, tabComponentList);
                }
            } else {
                String bindTable = config.getBindTable();
                if (StrUtil.isEmpty(bindTable)) {
                    continue;
                }
                List<ComponentConfig> componentConfigList = targetMap.computeIfAbsent(bindTable, k -> new ArrayList<>());
//                config.setBindField(config.getBindField());
//                config.setBindStartTime(config.getBindStartTime());
//                config.setBindEndTime(config.getBindEndTime());
                componentConfigList.add(config);
            }
        }
    }

    public static List<ComponentConfig> getFormComponentListWithoutLayout(List<ComponentConfig> sourceList) {
        List<ComponentConfig> resultList = new ArrayList<>();
        for (ComponentConfig config : sourceList) {
            String type = config.getType();
            if (StringUtils.equalsIgnoreCase(type, ComponentTypeConstant.TAB)
                    || StringUtils.equalsIgnoreCase(type, ComponentTypeConstant.CARD)
                    || StringUtils.equalsIgnoreCase(type, ComponentTypeConstant.GRID)) {
                List<LayoutOptionModel> tabList = config.getLayout();
                for (LayoutOptionModel tab : tabList) {
                    List<ComponentConfig> tabComponentList = tab.getList();
                    resultList.addAll(getFormComponentListWithoutLayout(tabComponentList));
                }
            } else if (StringUtils.equalsIgnoreCase(type, ComponentTypeConstant.OPINION)) {
                continue;
            } else {
                resultList.add(config);
            }
        }
        return resultList;
    }

    public static List<ComponentConfig> getFormComponentListWithMain(List<ComponentConfig> sourceList) {
        List<ComponentConfig> resultList = new ArrayList<>();
        for (ComponentConfig config : sourceList) {
            String type = config.getType();
            if (StringUtils.equalsIgnoreCase(type, ComponentTypeConstant.SUB_FORM) || StrUtil.equalsIgnoreCase(type, ComponentTypeConstant.ONE_FOR_ONE_FORM)) {
                continue;
            } else if (StringUtils.equalsIgnoreCase(type, ComponentTypeConstant.TAB)
                    || StringUtils.equalsIgnoreCase(type, ComponentTypeConstant.CARD)
                    || StringUtils.equalsIgnoreCase(type, ComponentTypeConstant.GRID)) {
                List<LayoutOptionModel> tabList = config.getLayout();
                for (LayoutOptionModel tab : tabList) {
                    List<ComponentConfig> tabComponentList = tab.getList();
                    resultList.addAll(getFormComponentListWithMain(tabComponentList));
                }
            } else {
                String bindTable = config.getBindTable();
                if (StrUtil.isEmpty(bindTable)) {
                    continue;
                }
                resultList.add(config);
            }
        }
        return resultList;
    }
}
