package com.xjrsoft.module.organization.service.impl;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.dingtalk.api.response.OapiV2DepartmentListsubResponse;
import com.dingtalk.api.response.OapiV2UserListResponse;
import com.xjrsoft.common.enums.DeleteMark;
import com.xjrsoft.common.enums.EnabledMark;
import com.xjrsoft.common.utils.DingtalkUtil;
import com.xjrsoft.module.organization.entity.Department;
import com.xjrsoft.module.organization.entity.User;
import com.xjrsoft.module.organization.service.DingtalkService;
import com.xjrsoft.module.organization.service.IDepartmentService;
import com.xjrsoft.module.organization.service.IUserService;
import lombok.AllArgsConstructor;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;
import java.util.*;
import java.util.stream.Collectors;

@Service
@AllArgsConstructor
public class DingtalkServiceImpl implements DingtalkService {


    public List<OapiV2DepartmentListsubResponse.DeptBaseResponse> deptResponselist;

    private final DingtalkUtil dingtalkUtil;

    private final IUserService userService;
    private final IDepartmentService departmentService;


    @Override
    public boolean updateInfo() throws Exception {
        deptResponselist.clear();
        this.getdept(1L);
        return this.update();
    }

    private boolean update() throws Exception {
        //更新部门信息
        List<Department> departmentList = departmentService.list(Wrappers.<Department>query().lambda().eq(Department::getEnabledMark, EnabledMark.ENABLED.getCode()).eq(Department::getDeleteMark, DeleteMark.NODELETE.getCode()));
        ArrayList<Department> updateDeparts = new ArrayList<>();
        ArrayList<Department> addDeparts = new ArrayList<>();

        out:for (OapiV2DepartmentListsubResponse.DeptBaseResponse deptBaseResponse  : deptResponselist) {
            for (Department department : departmentList) {
                if (department.getId().equals(deptBaseResponse.getDeptId())) {
                    department.setName(deptBaseResponse.getName());
                    department.setParentId(deptBaseResponse.getParentId());
                    updateDeparts.add(department);
                    continue out;
                }
            }

            Department department = new Department();
            department.setId(deptBaseResponse.getDeptId());
            department.setName(deptBaseResponse.getName());
            department.setParentId(deptBaseResponse.getParentId());
            addDeparts.add(department);
        }
        try {
            departmentService.saveBatch(addDeparts);
            departmentService.updateBatchById(updateDeparts);
        }catch (DuplicateKeyException e){
            throw  new Exception("用户和部门信息已更新到其他公司中!!!");
        }
        //更新用户
        List<OapiV2UserListResponse.ListUserResponse> userResponses = new ArrayList<>();
        //ding'ding中的用户信息
        for (OapiV2DepartmentListsubResponse.DeptBaseResponse deptBaseResponse : deptResponselist) {
            userResponses.addAll(dingtalkUtil.getDepartmentUser(deptBaseResponse.getDeptId()));
        }

        for (int i = 0; i < userResponses.size(); i++) {
            if ("".equals(userResponses.get(i).getMobile())) {
                userResponses.remove(i);
            }
        }
        //获取数据库中的用户信息
        List<User> dbUsers = userService.list(Wrappers.<User>query().lambda().eq(User::getEnabledMark, EnabledMark.ENABLED.getCode()).eq(User::getDeleteMark, DeleteMark.NODELETE.getCode()));

        Map<String, User>dbUsersMap = dbUsers.stream().filter(v->StrUtil.isNotBlank(v.getMobile())).collect(Collectors.toMap(v -> v.getMobile(), v -> v));

        Map<String, OapiV2UserListResponse.ListUserResponse> userResponsesMap =new HashMap<>();
        if(userResponses.size()>0) {
            userResponsesMap=userResponses.stream().collect(Collectors.toMap(v -> v.getMobile(), v -> v));
        }
        //2.更新部门下的用户信息；
        //更新用户信息
        ArrayList<User> updateUsers = new ArrayList<>();
        Set<String> subtractIds = CollUtil.intersectionDistinct(dbUsersMap.keySet(), userResponsesMap.keySet());
        if (subtractIds.size() > 0) {
            for (String subtractId : subtractIds) {
                User user = dbUsersMap.get(subtractId);
                OapiV2UserListResponse.ListUserResponse dinguserResponse = userResponsesMap.get(subtractId);
                user.setName(StrUtil.isNotBlank(dinguserResponse.getName())?dinguserResponse.getName():"未知姓名");
                user.setAvatar(StrUtil.isNotBlank(dinguserResponse.getAvatar())?dinguserResponse.getAvatar():"未知头像");
                user.setEmail(StrUtil.isNotBlank(dinguserResponse.getEmail())?dinguserResponse.getEmail():"未知邮箱");
                user.setMobile(StrUtil.isNotBlank(dinguserResponse.getMobile())?dinguserResponse.getMobile():"未知号码");
                List<Long> department = dinguserResponse.getDeptIdList();
                for (Long aLong : department) {
                    user.setDepartmentId(aLong);
                }
                //    user.setDepartmentId(weChatUserDto.getDepartment());
                updateUsers.add(user);
            }
        }

        //添加数据库中没有的用户
        List<String> subtractIds2 = CollUtil.subtractToList(userResponsesMap.keySet(), dbUsersMap.keySet());
        if (subtractIds2.size() > 0) {
            for (String subtractId : subtractIds2) {
                User user = new User();
                OapiV2UserListResponse.ListUserResponse dinguserResponse = userResponsesMap.get(subtractId);
                user.setName(StrUtil.isNotBlank(dinguserResponse.getName())?dinguserResponse.getName():"未知姓名");
                user.setAvatar(StrUtil.isNotBlank(dinguserResponse.getAvatar())?dinguserResponse.getAvatar():"未知头像");
                user.setEmail(StrUtil.isNotBlank(dinguserResponse.getEmail())?dinguserResponse.getEmail():"未知邮箱");
                user.setMobile(StrUtil.isNotBlank(dinguserResponse.getMobile())?dinguserResponse.getMobile():"未知号码");
                List<Long> department = dinguserResponse.getDeptIdList();
                for (Long aLong : department) {
                    user.setDepartmentId(aLong);
                }
                userService.save(user);
            }
        }
        userService.updateBatchById(updateUsers);
        return true;
    }

    private void getdept(Long deptId) throws Exception {
        List<OapiV2DepartmentListsubResponse.DeptBaseResponse> departmentList = dingtalkUtil.getDepartmentList(deptId);
        if (departmentList==null||departmentList.size() == 0) {
            return;
        }
        deptResponselist.addAll(departmentList);
        for (OapiV2DepartmentListsubResponse.DeptBaseResponse deptBaseResponse : departmentList) {
            getdept(deptBaseResponse.getDeptId());
        }
    }
}
