package com.xjrsoft.module.student.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.yulichang.wrapper.MPJLambdaWrapper;
import com.xjrsoft.common.model.result.R;
import com.xjrsoft.common.page.ConventPage;
import com.xjrsoft.common.page.PageOutput;
import com.xjrsoft.common.utils.VoToColumnUtil;
import com.xjrsoft.module.student.dto.AddBaseStudentUserDto;
import com.xjrsoft.module.student.dto.BaseStudentUserPageDto;
import com.xjrsoft.module.student.dto.UpdateBaseStudentUserDto;
import com.xjrsoft.module.student.entity.BaseStudent;
import com.xjrsoft.module.student.entity.BaseStudentSchoolRoll;
import com.xjrsoft.module.student.entity.BaseStudentUser;
import com.xjrsoft.module.student.service.IStudentManagerService;
import com.xjrsoft.module.student.vo.BaseStudentUserPageVo;
import com.xjrsoft.module.student.vo.BaseStudentUserVo;
import com.xjrsoft.module.system.entity.DictionaryDetail;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

@RestController
@RequestMapping("/student" + "/studentmanager")
@Api(value = "/student" + "/studentmanager", tags = "学生基本信息管理代码")
@AllArgsConstructor
public class StudentManagerController {

    private final IStudentManagerService studentManagerService;


    @GetMapping(value = "/page")
    @ApiOperation(value = "学生列表(分页)")
    @SaCheckPermission("studentmanager:detail")
    public R page(@Valid BaseStudentUserPageDto dto) {

        MPJLambdaWrapper<BaseStudentUser> queryWrapper = new MPJLambdaWrapper<>();
        queryWrapper
                .disableSubLogicDel()
                .like(StrUtil.isNotBlank(dto.getUserName()), BaseStudentUser::getUserName, dto.getUserName())
                .like(StrUtil.isNotBlank(dto.getMobile()), BaseStudentUser::getMobile, dto.getMobile())
                .like(StrUtil.isNotBlank(dto.getName()), BaseStudentUser::getName, dto.getName())
                .like(StrUtil.isNotBlank(dto.getEmail()), BaseStudentUser::getEmail, dto.getEmail())
                .orderByDesc(BaseStudentUser::getId)
                .select(BaseStudentUser::getId)
                .select(BaseStudentUser.class, x -> VoToColumnUtil.fieldsToColumns(BaseStudentUserPageVo.class).contains(x.getProperty()))
                .select(BaseStudentSchoolRoll::getStudyYear)
                .select("a.`name` as grade_id")
                .select("b.`name` as major_set_id")
                .select("c.`name` as class_id")
                .innerJoin(BaseStudent.class, BaseStudent::getUserId, BaseStudentUser::getId)
                .leftJoin(BaseStudentSchoolRoll.class, BaseStudentSchoolRoll::getUserId, BaseStudentUser::getId)
                .leftJoin(DictionaryDetail.class, DictionaryDetail::getCode, BaseStudentSchoolRoll::getArchivesStatus, ext -> ext.selectAs(DictionaryDetail::getName, BaseStudentUserPageVo::getArchivesStatus))
                .leftJoin("base_grade a on a.id = t2.grade_id")
                .leftJoin("base_major_set b on b.id = t2.major_set_id")
                .leftJoin("base_class c on c.id = t2.class_id")

                .selectAsClass(BaseStudent.class, BaseStudentUserPageVo.class);

        IPage<BaseStudentUser> page = studentManagerService.page(ConventPage.getPage(dto), queryWrapper);
        PageOutput<BaseStudentUserPageVo> pageOutput = ConventPage.getPageOutput(page, BaseStudentUserPageVo.class);
        return R.ok(pageOutput);
    }

    @GetMapping(value = "/info")
    @ApiOperation(value = "根据id查询学生信息")
    @SaCheckPermission("studentmanager:detail")
    public R info(@RequestParam Long id) {
        BaseStudentUser baseStudentUser = studentManagerService.getByIdDeep(id);
        if (baseStudentUser == null) {
            return R.error("找不到此数据！");
        }
        return R.ok(BeanUtil.toBean(baseStudentUser, BaseStudentUserVo.class));
    }


    @PostMapping
    @ApiOperation(value = "新增学生")
    @SaCheckPermission("studentmanager:add")
    public R add(@Valid @RequestBody AddBaseStudentUserDto dto) {
        BaseStudentUser baseStudentUser = BeanUtil.toBean(dto, BaseStudentUser.class);
        return R.ok(studentManagerService.add(baseStudentUser));

    }

    @PutMapping
    @ApiOperation(value = "修改学生")
    @SaCheckPermission("studentmanager:edit")
    public R update(@Valid @RequestBody UpdateBaseStudentUserDto dto) {

        BaseStudentUser baseStudentUser = BeanUtil.toBean(dto, BaseStudentUser.class);
        return R.ok(studentManagerService.update(baseStudentUser));

    }

    @DeleteMapping
    @ApiOperation(value = "删除")
    @SaCheckPermission("studentmanager:delete")
    public R delete(@Valid @RequestBody List<Long> ids) {
        return R.ok(studentManagerService.delete(ids));
    }

    @GetMapping(value = "/class")
    @ApiOperation(value = "获取当前学生的班主任")
    public R studentClass() {
        Long userId = StpUtil.getLoginIdAsLong();
        return R.ok(studentManagerService.getStudentClass(userId));
    }
}
