package com.xjrsoft.module.teacher.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.yulichang.wrapper.MPJLambdaWrapper;
import com.xjrsoft.common.model.result.R;
import com.xjrsoft.common.page.ConventPage;
import com.xjrsoft.common.page.PageOutput;
import com.xjrsoft.common.utils.VoToColumnUtil;
import com.xjrsoft.module.system.entity.DictionaryDetail;
import com.xjrsoft.module.system.service.IDictionarydetailService;
import com.xjrsoft.module.teacher.dto.AddXjrUserDto;
import com.xjrsoft.module.teacher.dto.UpdateXjrUserDto;
import com.xjrsoft.module.teacher.dto.XjrUserPageDto;
import com.xjrsoft.module.teacher.entity.BaseTeacher;
import com.xjrsoft.module.teacher.entity.XjrUser;
import com.xjrsoft.module.teacher.service.ITeacherbaseManagerService;
import com.xjrsoft.module.teacher.vo.XjrUserPageVo;
import com.xjrsoft.module.teacher.vo.XjrUserVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/**
* @title: 教职工基本信息管理
* @Author 管理员
* @Date: 2023-08-08
* @Version 1.0
*/
@RestController
@RequestMapping("/teacher" + "/teacherbasemanager")
@Api(value = "/teacher"  + "/teacherbasemanager",tags = "教职工基本信息管理代码")
@AllArgsConstructor
public class TeacherbaseManagerController {


    private final ITeacherbaseManagerService teacherbaseManagerService;
    private final IDictionarydetailService dictionarydetailService;

    @GetMapping(value = "/page")
    @ApiOperation(value="XjrUser列表(分页)")
    @SaCheckPermission("teacherbasemanager:detail")
    public R page(@Valid XjrUserPageDto dto){

        MPJLambdaWrapper<XjrUser> queryWrapper = new MPJLambdaWrapper<>();
        queryWrapper
                .disableSubLogicDel()
                .like(StrUtil.isNotBlank(dto.getUserName()),XjrUser::getUserName,dto.getUserName())
                .like(StrUtil.isNotBlank(dto.getMobile()),XjrUser::getMobile,dto.getMobile())
                .like(StrUtil.isNotBlank(dto.getName()),XjrUser::getName,dto.getName())
                .like(StrUtil.isNotBlank(dto.getEmail()),XjrUser::getEmail,dto.getEmail())
                    .orderByDesc(XjrUser::getId)
                .select(XjrUser::getId)
                .select(XjrUser.class,x -> VoToColumnUtil.fieldsToColumns(XjrUserPageVo.class).contains(x.getProperty()))
                .innerJoin(BaseTeacher.class,BaseTeacher::getUserId,XjrUser::getId)
                .leftJoin(DictionaryDetail.class,DictionaryDetail::getCode,BaseTeacher::getJobState,ext->ext.selectAs(DictionaryDetail::getName, XjrUserPageVo::getJobState))
                .leftJoin(DictionaryDetail.class,DictionaryDetail::getCode,XjrUser::getCredentialType,ext->ext.selectAs(DictionaryDetail::getName, XjrUserPageVo::getCredentialType))
                .leftJoin(DictionaryDetail.class,DictionaryDetail::getCode,BaseTeacher::getEmployWay,ext->ext.selectAs(DictionaryDetail::getName, XjrUserPageVo::getEmployWay))
                .selectAsClass(BaseTeacher.class, XjrUserPageVo.class);

        IPage<XjrUser> page = teacherbaseManagerService.page(ConventPage.getPage(dto), queryWrapper);
        PageOutput<XjrUserPageVo> pageOutput = ConventPage.getPageOutput(page, XjrUserPageVo.class);

        return R.ok(pageOutput);
    }

    @GetMapping(value = "/info")
    @ApiOperation(value="根据id查询XjrUser信息")
    @SaCheckPermission("teacherbasemanager:detail")
    public R info(@RequestParam Long id){
        XjrUser xjrUser = teacherbaseManagerService.getByIdDeep(id);
        if (xjrUser == null) {
           return R.error("找不到此数据！");
        }
        return R.ok(BeanUtil.toBean(xjrUser, XjrUserVo.class));
    }


    @PostMapping
    @ApiOperation(value = "新增XjrUser")
    @SaCheckPermission("teacherbasemanager:add")
    public R add(@Valid @RequestBody AddXjrUserDto dto){
        XjrUser xjrUser = BeanUtil.toBean(dto, XjrUser.class);
        return R.ok(teacherbaseManagerService.add(xjrUser));

    }

    @PutMapping
    @ApiOperation(value = "修改XjrUser")
    @SaCheckPermission("teacherbasemanager:edit")
    public R update(@Valid @RequestBody UpdateXjrUserDto dto){

        XjrUser xjrUser = BeanUtil.toBean(dto, XjrUser.class);
        return R.ok(teacherbaseManagerService.update(xjrUser));

    }

    @DeleteMapping
    @ApiOperation(value = "删除")
    @SaCheckPermission("teacherbasemanager:delete")
    public R delete(@Valid @RequestBody List<Long> ids){
        return R.ok(teacherbaseManagerService.delete(ids));

    }

}